#ifndef Photophoria_AlbumCanvas_h
#define Photophoria_AlbumCanvas_h

#include "Browser.h"

#include <wx/window.h>
#include <wx/timer.h>
#include <wx/dnd.h>
//-----------------------------------------------------------------------------
// AlbumCanvas
//-----------------------------------------------------------------------------
class GaugeControler;

namespace browser 
{
	class Browser;
	struct AlbumInfo;
	struct FileEntry;
}

class MyFrame;
class AlbumView;
class AlbumCanvas;
class AlbumInfoBook;
class AlbumController;

class AlbumDropTarget : public wxTextDropTarget
{
public:
	AlbumDropTarget(AlbumCanvas * pCanvas) : m_pCanvas(pCanvas) { }
	
	// Drag'n'drop (tags)
	virtual bool OnDropText(wxCoord x, wxCoord y, const wxString& data);
	virtual wxDragResult OnDragOver(wxCoord x, wxCoord y, wxDragResult def);

protected:
	AlbumCanvas * m_pCanvas;
};


// TODO: put in a wxland namespace
class AlbumCanvas: public wxWindow, public browser::BrowserCallback
{
public:
    AlbumCanvas( wxWindow *parent, GaugeControler * gg, 
			     wxWindowID, const wxPoint &pos, const wxSize &size );
	virtual ~AlbumCanvas();

	void SetController(AlbumController * pController);

	browser::FileEntry * GetCurrentFile();
	const browser::ClipboardType & GetSelection() const;

    void Display(wxDC & dc, int min_y=0, int max_y=-1, int y1=-1);
    
	void SetAlbums(const std::vector <browser::AlbumInfo * > & );
	void SelectAlbum (browser::AlbumInfo * pAlbum);
	void SetAlbumsThumbnailSize(int x);

    void LoadAlbum();
	void NextPicture(bool previous);



	// Implementation of BrowserCallback:
	virtual void BeginBrowsingAlbum(browser::AlbumInfo *);
    void LoadingThumbnail(browser::AlbumInfo *, size_t i);
	virtual void FinishBrowsingAlbum(browser::AlbumInfo *);


	static const unsigned int SCROLLING = 16;
	enum Activity { IDLE=0, FADING = 1, MANUAL_SCROLLING=SCROLLING, AUTO_SCROLLING};

	static const int thumbnailSizes[]; 

protected:
	friend class AlbumDropTarget;
	friend class AlbumController;
	void DrawScrollbar(wxDC & dc, bool fast);

	// events:
	void OnPaint( wxPaintEvent &event );

	void OnClick(wxMouseEvent & event);
	void OnRightClick(wxMouseEvent & event);
	void OnDoubleClick(wxMouseEvent& event);
	void OnEraseBackGround(wxEraseEvent& event);
	void OnResize(wxSizeEvent & event);
	void MouseMotion(wxMouseEvent & event);
	void OnScrollTimer(wxTimerEvent & event);

	//event helper:
	// returns the album and entry located under the mouse. 
	// can return nil for one or both of its return values.
	void GetEntry(const wxMouseEvent& event, AlbumView * &pAlbumView, browser::AlbumEntry* & pAlbumEntry);
	void SelectionClick(wxMouseEvent& event);

	// Utility to restrict the scrolling value to an acceptable range
	int AlbumCanvas::BoundScroll(int y);

	// Perform the graphical scrolling + refresh request
	void LocalScroll(int y);
	int GetLocalScrollPosition() const;

	// 3 ways to change the position : manual scrolling, auto scrolling or fading
	void StartManualScrolling(int eventY);
	void AlbumCanvas::PageUpDown(bool goingUp);
	void JumpTo(int targetY);


	int m_localScrollPosition;

	AlbumController * m_pController;
	GaugeControler * m_pGaugeControler;
	MyFrame * m_pMainFrame; // TODO: turn into an interface (used for the Album ListBox)
	typedef std::vector<AlbumView *> AlbumViews;
	AlbumViews m_albums;
	AlbumView * m_pAlbumView;
	AlbumView * m_pNextAlbumView; // used by the Browser Callback
	base::size m_currentHeight;
	browser::AlbumEntry * m_pCurrentFile;
	wxTimer * m_pTimer;
	browser::ClipboardType m_clipboard;

	int thumb_size;


private:
    DECLARE_EVENT_TABLE()
};

#endif
