#include <stdafx.h>
#include "AlbumController.h"
#include "AlbumCanvas.h"
#include "AlbumView.h"
#include "AlbumInfoBook.h"
#include "Persistence.h"
#include "Browser.h"
#include "MainFrame.h"
#include "ThemeDrawing.h"

#include <wx/stattext.h>
#include <wx/sysopt.h>
#include <wx/treectrl.h>

#include "../images/bitmaps.h"

const int ID_BTN_NEW = 401;
const int ID_BTN_OPTIONS = 404;

static const int ID_MENU_SORT1 = 1121;
static const int ID_MENU_SORT2 = 1122;
static const int ID_MENU_SORT3 = 1123;
static const int ID_MENU_SORT4 = 1124;

static const int ID_MENU_SIZE1 = 1131;
static const int ID_MENU_SIZE5 = ID_MENU_SIZE1+4;

const int ID_ALBUM_LIST = 150;



BEGIN_EVENT_TABLE(AlbumController,wxEvtHandler)
// Note that some Events are connected by hand in the GetPanel method to get around some 
// wxWindows oddities (namely : a crash in the FocusEvent handling :) ).
// At some point, we should move them back here, by making this class inherit from a wxWindow
// rather than from an EventHandler. 
  EVT_MENU_RANGE(ID_MENU_SORT1, ID_MENU_SORT4, AlbumController::OnSortCommand)
  EVT_MENU_RANGE(ID_MENU_SIZE1, ID_MENU_SIZE5, AlbumController::OnThumbnailSizeCommand)
END_EVENT_TABLE()


template <typename T>
struct AlbumEntrySorter
{
	AlbumEntrySorter(T browser::FileEntry::*member) : m(member) { };
	T browser::FileEntry::*m;


	bool operator ()(const browser::AlbumEntry * lhs, const browser::AlbumEntry * rhs)
	{
		return lhs->pFileEntry->*m < rhs->pFileEntry->*m;
	}
};

template <typename T>
void Sorter(AlbumView * pAlbum, T sorter)
{
	std::sort(pAlbum->m_entries.begin(), pAlbum->m_entries.end(), sorter);
	for (size_t i=0; i<pAlbum->m_entries.size(); ++i)
		pAlbum->m_entries[i]->index=i;
}

AlbumController::AlbumController(MyFrame * pFrame, AlbumCanvas * pCanvas)
	: m_pFrame(pFrame), m_pCanvas(pCanvas)
{
	currentOrdering = FOLDER;
}

AlbumCanvas * AlbumController::GetCanvas()
{ 
	return m_pCanvas; 
}

void AlbumController::SelectAlbum(browser::AlbumInfo * pAlbum) 
{ 
	m_pCanvas->SelectAlbum(pAlbum); 
}

void AlbumController::SetOrderingMode(Ordering order)
{
	if (currentOrdering!=order)
		SwitchOrderingMode();
}

void AlbumController::SwitchOrderingMode()
{
	if (currentOrdering==FOLDER)
		currentOrdering = CALENDAR;
	else
		currentOrdering = FOLDER;

	Persistence::Ordering ordering = (currentOrdering == CALENDAR) ? Persistence::CALENDAR : Persistence::FOLDER;
	m_pCanvas->SetAlbums(Persistence::Instance()->GetAlbums(ordering));

}

void AlbumController::NewAlbum( wxCommandEvent &WXUNUSED(event) )
{
	//TODO: move the code here
	m_pCanvas->LoadAlbum();
}


void AlbumController::AlbumsOptions( wxCommandEvent &WXUNUSED(event) )
{
	PopupMenu();
}

wxPanel * AlbumController::GetAlbumPanel(wxPanel * pParent)
{
	wxPanel * pAlbumPanel = new wxPanel (pParent, -1, wxPoint(0,0), wxSize(theme::panelWidth,200) );
	theme::SetBackgroundColor(pAlbumPanel);


	m_pAlbumList = new wxTreeCtrl (pAlbumPanel, ID_ALBUM_LIST, wxPoint(theme::panelLeftEdge,400), wxSize(theme::panelWidth-8,200),wxSUNKEN_BORDER | wxTR_FULL_ROW_HIGHLIGHT | wxTR_NO_LINES | wxTR_LINES_AT_ROOT | wxTR_HAS_BUTTONS );
	static wxFont * f1 = wxFont::New(10, wxFONTFAMILY_DECORATIVE  , wxFONTFLAG_ANTIALIASED);
	m_pAlbumList->SetFont(*f1);

	wxTreeItemId id1 = m_pAlbumList->AddRoot("All Albums");
	m_pAlbumList->SetIndent(15);
	m_pAlbumList->SetItemBold( id1);
	m_pAlbumList->Expand( id1);

	wxSystemOptions::SetOption(wxT("msw.remap"), 0);
	wxToolBar * pToolBar = new wxToolBar(pAlbumPanel, -1, wxDefaultPosition, wxSize(theme::panelWidth,50),  wxTB_FLAT | wxTB_TEXT | wxTB_HORIZONTAL | wxNO_BORDER);
	theme::SetBackgroundColor(pToolBar);
	pToolBar->SetToolBitmapSize(wxSize(22,22));
	pToolBar->AddTool(ID_BTN_NEW, "New", GET_BITMAP(22_add));
	pToolBar->AddTool(ID_BTN_OPTIONS, "Options", GET_BITMAP(22_configure));
	pToolBar->Realize();

	m_pInfoBook = new AlbumInfoBook(pAlbumPanel, this, -1, wxPoint(0,0), wxSize(theme::panelWidth-8,400), wxNO_BORDER | wxCLIP_CHILDREN );

	wxBoxSizer * pAlbumPanelSizer = new wxBoxSizer(wxVERTICAL);
	pAlbumPanelSizer->Add(pToolBar, 0, wxALL, 0);
	pAlbumPanelSizer->Add(m_pAlbumList, 2 /* stretch*/ , wxGROW | wxALL, 4);
	pAlbumPanelSizer->Add(m_pInfoBook, 0, wxGROW | wxALL, 3);
	pAlbumPanel->SetSizer(pAlbumPanelSizer);
	pAlbumPanelSizer->Layout();

	m_pAlbumList->Connect( ID_ALBUM_LIST,    wxEVT_COMMAND_TREE_SEL_CHANGED,    wxTreeEventHandler(AlbumController::OnAlbumClick), 0, this );
	pToolBar->Connect( ID_BTN_NEW, wxEVT_COMMAND_TOOL_CLICKED,    wxCommandEventHandler(AlbumController::NewAlbum), 0, this );
	pToolBar->Connect( ID_BTN_OPTIONS, wxEVT_COMMAND_TOOL_CLICKED,    wxCommandEventHandler(AlbumController::AlbumsOptions), 0, this );

	return pAlbumPanel; 
}

void AlbumController::FinishLoad()
{
//	m_pAlbumList->PushEventHandler(this);
}

static std::map <browser::AlbumInfo::Id, wxTreeItemId> albumToTreeId;

class AlbumTreeData : public wxTreeItemData
{
public:
	browser::AlbumInfo * pAlbum;
};


void AlbumController::FillAlbumList()
{
	const std::vector<browser::AlbumInfo * > & albums = 
				Persistence::Instance()->GetAlbums(Persistence::FOLDER);

	m_pAlbumList->Freeze();
	for (size_t i=0; i<albums.size(); ++i)
		AddAlbum(albums[i]);
	m_pAlbumList->Thaw();
	m_pAlbumList->Expand(m_pAlbumList->GetRootItem());
}


void AlbumController::OnAlbumClick( wxTreeEvent & event)
{
	wxTreeItemId id = event.GetItem();
	AlbumTreeData * pATD = static_cast<AlbumTreeData *> (m_pAlbumList->GetItemData(id));
	if (pATD)
	{
		SetOrderingMode(AlbumController::FOLDER);
		SelectAlbum(pATD->pAlbum);
	}

}

void AlbumController::AddAlbum(browser::AlbumInfo * pAlbum)
{
	assert (pAlbum);
	if (!m_pAlbumList)
		return;

	wxTreeItemId parentTreeId = albumToTreeId[pAlbum->parentId];

	if (parentTreeId==wxTreeItemId(long(0)))
		parentTreeId = m_pAlbumList->GetRootItem();
	AlbumTreeData * pData = new AlbumTreeData();
	pData->pAlbum = pAlbum;
	const char * name = pAlbum->name.c_str();
	albumToTreeId[pAlbum->albumId] = 
		m_pAlbumList->AppendItem(parentTreeId, name, -1, -1, pData);
}

void AlbumController::UpdateSelection()
{
	// Set the infos in the little box
	m_pInfoBook->UpdateSelectionInfo();

	// Update the calendar
	if (m_pCanvas->GetCurrentFile())
		m_pInfoBook->UpdateCalendar(m_pCanvas->GetCurrentFile()->pictureDate);

	// Update the EXIF Info
	m_pInfoBook->UpdateExifTree();
}

void AlbumController::OnSortCommand(wxCommandEvent & event)
{
	for (size_t i=0; i<m_pCanvas->m_albums.size(); ++i)
	{
		AlbumView * pAlbum = m_pCanvas->m_albums[i];
		if (event.m_id==ID_MENU_SORT1)
			Sorter(pAlbum, AlbumEntrySorter<time_t> (&browser::FileEntry::pictureDate));
		else if (event.m_id==ID_MENU_SORT2)
			Sorter(pAlbum, AlbumEntrySorter<std::string> (&browser::FileEntry::name));
		else
			Sorter(pAlbum, AlbumEntrySorter<size_t> (&browser::FileEntry::fileSize));
	}
	m_pCanvas->Refresh();

}

void AlbumController::OnThumbnailSizeCommand(wxCommandEvent & event)
{
	m_pCanvas->SetAlbumsThumbnailSize(AlbumCanvas::thumbnailSizes[event.m_id-ID_MENU_SIZE1]);
}

void AlbumController::ShowPicture()
{
	m_pFrame->ShowPicture();
}

void AlbumController::PopupMenu()
{
	wxMenu menu;

	bool concernsSelection = false;
	if (concernsSelection)
	{
		// Actions
		menu.Append(4001, "Edit/View\tENTER");
		menu.Append(4001, "Open With\tShift Enter");
		menu.AppendSeparator();

		// Images
		menu.Append(4001, "Rotate Left (ccw)");
		menu.Append(4001, "Rotate Right (cw)");
		menu.AppendSeparator();

		// Tags
		wxMenu * pTagsMenu = new wxMenu("Tags");
		wxMenu * pPeopleMenu = new wxMenu("");
		pPeopleMenu->AppendCheckItem(4001, "Me");
		pPeopleMenu->AppendCheckItem(4001, "You");
		pTagsMenu->Append(4001, "People", pPeopleMenu);//pop
		pTagsMenu->Append(4001, "Places");
		pTagsMenu->Append(4001, "Events");

		menu.Append(-1, "Tags", pTagsMenu);
		menu.AppendSeparator();
	}

	menu.Append(4001, "Batch Convert");
	menu.Append(4001, "Correct Date");// : Set Date/Set Date to be File/ Shift Date or Time (fuseau horaire)");
	menu.AppendSeparator();

	if (!concernsSelection)
	{
		// Album
		wxMenu * pThumbnailMenu = new wxMenu("");
		pThumbnailMenu->Append(ID_MENU_SIZE1, "Tiny");
		pThumbnailMenu->Append(ID_MENU_SIZE1+1, "Small");
		pThumbnailMenu->Append(ID_MENU_SIZE1+2, "Normal");
		pThumbnailMenu->Append(ID_MENU_SIZE1+3, "Large"); 
		pThumbnailMenu->Append(ID_MENU_SIZE1+4, "Huge"); 
		menu.Append(4001, "Thumbnail Size", pThumbnailMenu);

		wxMenu * pSortMenu = new wxMenu("");
		pSortMenu->Append(ID_MENU_SORT1, "Date"); 
		pSortMenu->Append(ID_MENU_SORT2, "Name"); 
		pSortMenu->Append(ID_MENU_SORT3, "Size"); 
		menu.Append(4001, "Sort by", pSortMenu);
	}
	/*
	AlbumView * pAlbum;
	browser::AlbumEntry * pEntry;
	GetEntry(event, pAlbum, pEntry);
	if (!pEntry)*/
	m_pCanvas->PopupMenu(&menu);
}

