// Intel-specific code (using inline ASM with MMX)
// This file is meant to be included from AlbumCanvas. 
// Consider it as an inline file. 


#ifdef USE_GDIPLUS
#define USE_FADER 1
#include <gdiplus.h>
using namespace Gdiplus;
#define NO_INLINE
#endif

#if defined(__GNUC__) && defined(__i386__)
#define USE_FADER 1
#define NO_INLINE  __attribute__ ((noinline))
#endif

#ifdef USE_FADER

class AlbumFader
{
public:
	AlbumFader(wxDC & dc, AlbumCanvas * pCanvas, int targetPos);
	virtual ~AlbumFader();

	// Fade with alpha going from 0 to 15
	virtual void Fade(wxDC & dc, int alpha);
protected:
	base::octet * ConvertToWindows(const wxImage & image);

	void calc2(unsigned char *dst, unsigned char *src, 
		   unsigned char *src2, size_t s, unsigned char x) NO_INLINE;


	base::octet * m_pStart, * m_pEnd, *m_pTarget;

#ifdef USE_GDIPLUS
	Bitmap * m_pBitmap;
#else
	wxImage * m_pImage;
#endif
};


AlbumFader::AlbumFader(wxDC & dc, AlbumCanvas * pCanvas, int targetPos)
{
	wxSize screen = dc.GetSize();

	wxMemoryDC bitmapDc1;
	wxMemoryDC bitmapDc2;

	wxBitmap bmpStart(screen.x, screen.y);
	bitmapDc1.SelectObject(bmpStart);
	pCanvas->Display(bitmapDc1, 0, screen.y);
	m_pStart = ConvertToWindows(bmpStart.ConvertToImage());

	wxBitmap bmpEnd(screen.x, screen.y);
	bitmapDc2.SelectObject(bmpEnd);
	pCanvas->Display(bitmapDc2, 0, screen.y, targetPos);
	m_pEnd = ConvertToWindows(bmpEnd.ConvertToImage());
	m_pTarget = ConvertToWindows(bmpEnd.ConvertToImage());

#ifdef USE_GDIPLUS
	size_t stride = (screen.x*3 + 3) & 0xFFFFFFFC;
	m_pBitmap = new Bitmap (screen.x, screen.y, stride, PixelFormat24bppRGB, m_pTarget)#else
	m_pImage = new wxImage (screen.x, screen.y, m_pTarget, true);
#endif

}

AlbumFader::~AlbumFader()
{
	delete m_pStart;
	delete m_pEnd;
	delete m_pTarget;
#ifdef USE_GDIPLUS
	delete m_pBitmap;
#else
	delete m_pImage;
#endif
}


base::octet * AlbumFader::ConvertToWindows(const wxImage & image)
{
#ifdef USE_GDIPLUS
	size_t stride= (image.GetWidth()*3 +3)& 0xFFFFFFFC;
#else
	size_t stride= image.GetWidth()*3;
#endif

	base::octet * pBuffer = new base::octet[image.GetHeight()*stride];

	base::octet * p = image.GetData();
	base::octet * p2 = pBuffer;

	for (int i=0; i<image.GetHeight(); ++i)
	{
		for (int j=0; j<image.GetWidth(); ++j)
		{
			p2[base::cRed]=p[0];
			p2[base::cGreen]=p[1];
			p2[base::cBlue]=p[2];
			p+=3, p2+=3;
		}
		p2+=stride-image.GetWidth()*3;
	}
	return pBuffer;
}


void AlbumFader::Fade(wxDC & dc, int x) 
{
#ifdef USE_GDIPLUS

  	Graphics screenGraphics((HDC) dc.GetHDC());

	Rect rect(0, 0, m_pBitmap->GetWidth(), m_pBitmap->GetHeight());
	BitmapData bitmapData;

	m_pBitmap->LockBits(&rect,  ImageLockModeWrite, PixelFormat24bppRGB, &bitmapData);

	calc2((base::octet*) bitmapData.Scan0, m_pStart, m_pEnd, bitmapData.Height*bitmapData.Stride, x);

	m_pBitmap->UnlockBits(&bitmapData);

	screenGraphics.DrawImage(m_pBitmap, 0, 0);
#else

	calc2((base::octet*) m_pImage->GetData(), m_pStart, m_pEnd, 
	      m_pImage->GetWidth()*m_pImage->GetHeight()*3/8 -4 , x);

	wxBitmap b(*m_pImage);
	dc.DrawBitmap(b, 0, 0, false);

#endif
}

// This nice routine has been written by chrisbk. Cheers dude.

#ifdef USE_GDIPLUS
void AlbumFader::calc2(unsigned char *dst, unsigned char *src, unsigned char *src2, size_t s, unsigned char x) 
{
    __int64 tx = (x<<16) | x;
    tx = tx <<32 | tx;

    _asm {
		pusha
        mov eax, src;
        mov esi, src2;
        mov edi, dst;
        mov ecx,s;
        shr ecx,3;
		sub ecx,2
        movq mm5, tx;
		xor ebx, ebx
        pxor mm4,mm4;

start:
        movq mm0,[eax+ebx];
        movq mm1,[esi+ebx];
		punpcklbw mm0, mm4;
        punpcklbw mm1, mm4;

		movq mm2,[eax+ebx+4];
        movq mm3,[esi+ebx+4];

		punpcklbw mm2, mm4;
        punpcklbw mm3, mm4;
        
		psubsw mm1,mm0;
        psubsw mm3,mm2;

		pmullw mm1,mm5;
        pmullw mm3,mm5;

		prefetch[eax + ebx+ 64];
		prefetch[esi + ebx+ 64];

	    psllq mm0,4;
	    psllq mm2,4;

        paddsw mm1, mm0;
        paddsw mm3, mm2;
        psrlw mm1,4;
        psrlw mm3,4;

        packuswb mm1,mm4;
        packuswb mm3,mm4;
        movd [edi+ebx],mm1;
        movd [edi+ebx+4],mm3;

		add ebx,8;
        dec ecx;
		jnz start;

        femms;
		popa

    }
}

#else
	static long long tx;

void AlbumFader::calc2(unsigned char *dst, unsigned char *src, unsigned char *src2, size_t s, unsigned char x)
{
tx  = (x<<16) | x;
    tx = tx <<32 | tx;

   __asm__ (
      "pusha \n"
      "mov %%ebx,%%esi \n"
      "mov %%edx, %%edi \n"
      "movq tx, %%mm5   \n"
      "xor %%ebx, %%ebx \n"
      "pxor %%mm4,%%mm4 \n"

    "start:\n"
      "movq (%%eax), %%mm0\n"
      "movq (%%esi), %%mm1\n"
      "punpcklbw %%mm4, %%mm0\n"
      "punpcklbw %%mm4, %%mm1\n"

      "movq 4(%%eax), %%mm2\n"
      "movq 4(%%esi), %%mm3\n"

      "punpcklbw %%mm4, %%mm2\n"
      "punpcklbw %%mm4, %%mm3\n"
        
      "psubsw %%mm0,%%mm1\n"
      "psubsw %%mm2,%%mm3\n"

      "pmullw %%mm5,%%mm1\n"
      "pmullw %%mm5,%%mm3\n"

      "prefetch 64(%%eax)\n"
      "prefetch 64(%%esi)\n"

      "psllq $4, %%mm0\n"
      "psllq $4, %%mm2\n"

      "paddsw %%mm0, %%mm1\n"
      "paddsw %%mm2, %%mm3\n"
      "add $8, %%eax\n"
      "add $8, %%ebx\n"
      "psrlw $4, %%mm1\n"
      "psrlw $4, %%mm3\n"

      "packuswb %%mm4,%%mm1\n"
      "packuswb %%mm4,%%mm3\n"
      "movd %%mm1, (%%edi)\n"
      "movd %%mm3, 4(%%edi)\n"

      "add $8, %%esi\n"
      "add $8, %%edi\n"
      "dec %%ecx\n"
      "jnz start\n"

      "femms\n"
      "popa\n"

      : // no output regs
      :  "a"(src), "b"(src2), "c"(s), "d"(dst)
         );
  
}

#endif // GDI+ or GCC ?

#endif // USE_FADER ?
