#include "stdafx.h"
#include "AlbumInfoBook.h"
#include "ThemeDrawing.h"
#include "AlbumCanvas.h"
#include "AlbumController.h"
#include "Persistence.h"

#include "../images/bitmaps.h"

#include <wx/toolbar.h>
#include <wx/treectrl.h>
#include <wx/calctrl.h>
#include <wx/dataobj.h>
#include <wx/dnd.h>
#include <wx/image.h>


#include <iostream>
#include <fstream>
#include <sstream>
#include <iomanip>


// With wxGTK2.5.5, we need to set the toolbar color ourselves :(
#ifdef __WXGTK__
#include <gtk/gtk.h>
#endif

const int ID_MODE_INFOS = 171;
const int ID_MODE_CALENDAR = 172;
const int ID_MODE_EXIF = 173;
const int ID_MODE_TAGS = 174;


const int ID_CALENDAR = 151;
const int ID_INFOS = 151;
const int ID_TAGS_TREE = 153;

const int ID_TAGS_MENU_ADD_GROUP=701;
const int ID_TAGS_MENU_ADD_TAG=702;
const int ID_TAGS_MENU_RENAME=703;
const int ID_TAGS_MENU_DELETE=705;
const int ID_TAGS_MENU_TOGGLE=705;


BEGIN_EVENT_TABLE(AlbumInfoBook,wxControl)
EVT_TOOL_RANGE(ID_MODE_INFOS, ID_MODE_TAGS, AlbumInfoBook::OnInfoPaneChange)

  EVT_CALENDAR_MONTH(ID_CALENDAR, AlbumInfoBook::OnCalendarMonthChange) 
  EVT_CALENDAR_YEAR(ID_CALENDAR, AlbumInfoBook::OnCalendarMonthChange) 
  EVT_CALENDAR_SEL_CHANGED(ID_CALENDAR, AlbumInfoBook::OnCalendarSelection) 

 EVT_TREE_ITEM_MENU(ID_TAGS_TREE, AlbumInfoBook::OnTagsRightClick) 
 EVT_TREE_ITEM_RIGHT_CLICK(ID_TAGS_TREE, AlbumInfoBook::OnTagsRightClick) 
 EVT_TREE_BEGIN_DRAG(ID_TAGS_TREE, AlbumInfoBook::BeginTagsDrag) 
 EVT_MENU_RANGE(ID_TAGS_MENU_ADD_GROUP, ID_TAGS_MENU_TOGGLE, AlbumInfoBook::OnTagsCommand)


END_EVENT_TABLE()


AlbumInfoBook::AlbumInfoBook (wxWindow *parent, AlbumController * pAlbumController, wxWindowID winid, const wxPoint& pos, const wxSize& size, long style)
: wxControl (parent, winid, pos, size, style),  m_selection(0), m_pAlbumController(pAlbumController)
{
    	theme::SetBackgroundColor(this);
	m_pToolBar = new wxToolBar(this, -1, wxDefaultPosition, wxSize(209, 54),  wxTB_FLAT | wxTB_TEXT | wxNO_BORDER | wxTB_HORIZONTAL);
	theme::SetBackgroundColor(m_pToolBar);
	m_pToolBar->SetToolBitmapSize(wxSize(22,22));
	m_pToolBar->SetThemeEnabled(false);		
 	m_pToolBar->SetBackgroundStyle(wxBG_STYLE_COLOUR);
	m_pToolBar->SetBackgroundColour(theme::bgColor);
	/*
   GtkStyle * style1 = gtk_style_new ();
   gdk_color_parse ("#F0F0FF", &style1->bg[GTK_STATE_NORMAL]);
   gdk_color_parse ("#FFF00F", &style1->fg[GTK_STATE_PRELIGHT]);
   gdk_color_parse ("#F80000", &style1->fg[GTK_STATE_ACTIVE]);
   gdk_color_parse ("#00FF00", &style1->fg[GTK_STATE_NORMAL]);

   gtk_widget_set_style ((GtkWidget *) m_pToolBar->m_toolbar, style1);
	*/


	m_pInfosPanel = GetInfoPanel();
	m_pCalendar = GetCalendar();
	m_pExifTree = GetExifTree();
	m_pTagsTree = GetTagsTree();

	AddPage(m_pInfosPanel, " Infos ", GET_BITMAP(22_info));
	AddPage(m_pCalendar, "Date", GET_BITMAP(22_vcalendar));
	AddPage(m_pExifTree, "EXIF", GET_BITMAP(22_camera));
	AddPage(m_pTagsTree, "Tags", GET_BITMAP(22_bookmark));

	m_pToolBar->Realize();
	DoGetBestSize();
	InvalidateBestSize();
	ChangeSelection(0);
}


void AlbumInfoBook::AddPage(wxWindow * page, const wxString &s, const wxBitmap &b)
{
	m_pages.push_back(page);
	if (m_pages.size()==1)
	  ;//	ChangeSelection(0);
	else
		page->Hide();
	m_pToolBar->AddRadioTool(ID_MODE_INFOS-1+m_pages.size(), s, b);
}


wxSize AlbumInfoBook::DoGetBestSize() const
{	
	wxSize bestSize;

	// iterate over all pages, get the largest width and height
	const size_t nCount = m_pages.size();
	for ( size_t nPage = 0; nPage < nCount; nPage++ )
	{
		wxWindow *pPage = m_pages[nPage];
		wxSize childBestSize(pPage->GetBestSize());

		if ( childBestSize.x > bestSize.x )
			bestSize.x = childBestSize.x;

		if ( childBestSize.y > bestSize.y )
			bestSize.y = childBestSize.y;
	}
	bestSize.y+=55;
	bestSize.x+=2;
	
	static bool first=true;
	if (first)
	  std::cout << " Best size: " << bestSize.x << "x" << bestSize.y << std::endl;
	first=false;
	CacheBestSize(bestSize);
	return bestSize;
}

void AlbumInfoBook::OnInfoPaneChange(wxCommandEvent &event)
{
	ChangeSelection (event.GetId()-ID_MODE_INFOS);
}

void AlbumInfoBook::ChangeSelection(int n)
{
	m_pages[m_selection]->Hide();

    wxWindow *page = m_pages[n];
	wxRect rect(wxPoint(0,55), GetSize());
	rect.height-=55;
   page->SetSize(rect);
    page->Show();
	m_selection = n;
	Refresh();
};


////////////////////////////////////////////////////////////////////////////////////////////
//
//    Infos
//
////////////////////////////////////////////////////////////////////////////////////////////


wxPanel * AlbumInfoBook::GetInfoPanel()
{
	wxPanel * pInfos = new wxPanel(this, -1,  wxPoint(0,55), wxSize(209,165), wxSUNKEN_BORDER);
	pInfos->SetBackgroundColour(wxColour(255,255,255));
	wxGridSizer * pInfoSizer = new wxFlexGridSizer(2, 2, 2);

	m_infoTexts.push_back(new wxStaticText(pInfos, -1, "Type: "));
	m_infoTexts.push_back(new wxStaticText(pInfos, -1, "Picture"));

	m_infoTexts.push_back(new wxStaticText(pInfos, -1, "Date: "));
	m_infoTexts.push_back(new wxStaticText(pInfos, -1, "February 2005"));

	m_infoTexts.push_back(new wxStaticText(pInfos, -1, "Size: "));
	m_infoTexts.push_back(new wxStaticText(pInfos, -1, "320x400"));

	m_infoTexts.push_back(new wxStaticText(pInfos, -1, "Bytes: "));
	m_infoTexts.push_back(new wxStaticText(pInfos, -1, "156 kilobytes"));

	for (size_t i=0; i<m_infoTexts.size(); ++i)
		if (!(i%2))
			pInfoSizer->Add(m_infoTexts[i], 0 , wxALIGN_RIGHT | wxLEFT | wxTOP | wxBOTTOM, 3);
		else
			pInfoSizer->Add(m_infoTexts[i], 0 , wxRIGHT | wxTOP | wxBOTTOM, 3);

	pInfos->SetSizer(pInfoSizer);
	pInfoSizer->Layout();

	return pInfos;
}



std::string GetPrintableSize(size_t size)
{
	std::stringstream s;
	s.precision(2);
	s.flags(std::ios_base::fixed);
	s.width(3);
	if (size<1024)
		s << size << " B.";
	else if (size<1024*1024)
		s << (size/1024.0) << " KB.";
	else 
		s << (size/1024/1024.0) << " MB.";

	return s.str();
}

std::string GetPrintableDate(time_t pt)
{

	tm * t = localtime(&pt);
	std::stringstream s;
	
	s << std::setw(2) <<  std::setfill('0') << t->tm_mday << "/" 
		<< std::setw(2) <<  std::setfill('0') << t->tm_mon+1 << "/" 
		<< std::setw(2) <<  std::setfill('0') << t->tm_year%100 << " - " 
		<< std::setw(2) <<  std::setfill('0') << t->tm_hour << ":" 
		<< std::setw(2) <<  std::setfill('0') << t->tm_min;

	return s.str();
}


void AlbumInfoBook::UpdateSelectionInfo()
{
	std::stringstream s;
	AlbumCanvas * pCanvas = m_pAlbumController->GetCanvas();

	const browser::ClipboardType & selection = pCanvas->GetSelection();
	m_pInfosPanel->Freeze();

	if (selection.empty())
	{
	}
	else if (selection.size()==1)
	{
		const browser::FileEntry * pF = pCanvas->GetCurrentFile();
		m_infoTexts[0]->SetLabel("Picture: ");
		m_infoTexts[1]->SetLabel(pF->name.c_str());
/*
		m_infoTexts[0]->SetLabel("Type: ");
		m_infoTexts[1]->SetLabel("Picture");
*/
		m_infoTexts[2]->SetLabel("Date: ");
		m_infoTexts[3]->SetLabel(GetPrintableDate(pF->pictureDate).c_str());

		std::stringstream s;
		s << pF->width << "x" << pF->height;
		m_infoTexts[4]->SetLabel("Size: ");
		m_infoTexts[5]->SetLabel(s.str().c_str());

		m_infoTexts[6]->SetLabel("Bytes: ");
		m_infoTexts[7]->SetLabel(GetPrintableSize(pF->fileSize).c_str());
	}
	else
	{
		unsigned long size = 0;
		for (browser::ClipboardType::const_iterator it=selection.begin();it!=selection.end(); ++it)
			size += (*it)->pFileEntry->fileSize;
	
		m_infoTexts[0]->SetLabel("Type: ");
		m_infoTexts[1]->SetLabel("Pictures");

		m_infoTexts[2]->SetLabel("Date: ");
		m_infoTexts[3]->SetLabel("1-Jan-70 to Now");

		std::stringstream s;
		s << selection.size() << " pictures";
		m_infoTexts[4]->SetLabel("Size: ");
		m_infoTexts[5]->SetLabel(s.str().c_str());

		m_infoTexts[6]->SetLabel("Bytes: ");
		m_infoTexts[7]->SetLabel(GetPrintableSize(size).c_str());
		
		
		s << " " << selection.size() << " pictures selected. " << GetPrintableSize(size);
	}

	m_pInfosPanel->GetSizer()->Layout();
	m_pInfosPanel->Thaw();

}



////////////////////////////////////////////////////////////////////////////////////////////
//
//    Calendar 
//
////////////////////////////////////////////////////////////////////////////////////////////

wxCalendarCtrl * AlbumInfoBook::GetCalendar()
{
	wxCalendarCtrl * pCalendar = new wxCalendarCtrl(this, ID_CALENDAR, wxDefaultDateTime, 
	wxPoint(theme::panelLeftEdge,400), wxSize(theme::panelWidth+2,200), 
	wxCAL_SHOW_SURROUNDING_WEEKS | wxCAL_MONDAY_FIRST );

	pCalendar->SetHeaderColours(wxColour(92,92,92), wxColour(224,224,224));
	return pCalendar;
}


void AlbumInfoBook::UpdateCalendar(time_t t)
{
	const Persistence::DateAlbums & months = Persistence::Instance()->GetMonths();
	const wxDateTime& date = m_pCalendar->GetDate();
	int m = date.GetMonth() + (date.GetYear()-1900)*12;

	Persistence::DateAlbums::const_iterator it = months.find(m);
	
	browser::AlbumInfo * pAlbum = it!=months.end() ? it->second : 0;;

	if (t!=-1)
	{
		wxDateTime d(t);
		m_pCalendar->SetDate(d);
	}

	std::map<int, int> days;

	if (pAlbum)
		for (size_t i =0; i<pAlbum->entries.size(); ++i)	
		{
			const browser::FileEntry * pFileEntry = pAlbum->entries[i];
			wxDateTime date2 (pFileEntry->pictureDate);
			size_t day= date2.GetDay();
			days[day]++;
		}

	for (int day=1; day<=31; ++day)
	{
		if (!days[day])
			m_pCalendar->SetAttr(day, 0);
		else
		{
			int b = 128;
			if (days[day]>=3)
				b = 255;
			wxCalendarDateAttr * pAttr = new wxCalendarDateAttr;
			pAttr->SetTextColour(wxColour(0,0,b));
			pAttr->SetBackgroundColour(wxColour(240,240,255));
			m_pCalendar->SetAttr(day, pAttr);
		}
	}
}



void AlbumInfoBook::OnCalendarMonthChange(wxCalendarEvent & event)
{
	UpdateCalendar();
}

void AlbumInfoBook::OnCalendarSelection(wxCalendarEvent & event)
{
	// TODO: this is a copy&paste from the UpdateCalendar code. To be factored out.
	const Persistence::DateAlbums & months = Persistence::Instance()->GetMonths();
	const wxDateTime& date = m_pCalendar->GetDate();
	int m = date.GetMonth() + (date.GetYear()-1900)*12;

	Persistence::DateAlbums::const_iterator it = months.find(m);
	if (it!=months.end())
	{
		m_pAlbumController->SetOrderingMode(AlbumController::CALENDAR);
		m_pAlbumController->SelectAlbum(it->second);
	}
}




////////////////////////////////////////////////////////////////////////////////////////////
//
//    EXIF Tree 
//
////////////////////////////////////////////////////////////////////////////////////////////

wxTreeCtrl * AlbumInfoBook::GetExifTree()
{
  wxTreeCtrl * pExifTree = new wxTreeCtrl (this, 141, wxPoint(theme::panelLeftEdge,400), wxSize(theme::panelWidth,65), wxTR_NO_LINES | wxSUNKEN_BORDER |wxTR_HIDE_ROOT );
	pExifTree->SetIndent(12);
	static wxFont * f2 = wxFont::New(7, wxFONTFAMILY_SWISS    , wxFONTFLAG_ANTIALIASED);
	pExifTree->SetFont(*f2);
	wxImageList * pIL = new wxImageList(16,16);
	pIL->Add(GET_BITMAP(16_item));
	pIL->Add(GET_BITMAP(16_camera));
	pIL->Add(GET_BITMAP(16_image));
	pIL->Add(GET_BITMAP(16_misc));
	pExifTree->AssignImageList(pIL);
	return pExifTree;
}

void AlbumInfoBook::UpdateExifTree()
{
	browser::FileEntry * pEntry = m_pAlbumController->GetCanvas()->GetCurrentFile();

	wxTreeCtrl * treeCtrl = m_pExifTree;

	treeCtrl->Freeze();
	treeCtrl->DeleteAllItems();
	wxTreeItemId idRoot = treeCtrl->AddRoot("All");

	if (!pEntry)
	{
		treeCtrl->AppendItem(idRoot, "No File Info available");
		treeCtrl->Thaw();
		return;
	}

	if (pEntry->topics.empty())
	{
		std::ifstream f;
		f.open(pEntry->path.c_str(), std::ios::binary);
		if (!f)
		{
			treeCtrl->AppendItem(idRoot, "File not available");
			treeCtrl->Thaw();
			return;
		}

		pEntry->pFileBuffer = new base::octet[pEntry->fileSize + 128];
		f.read((char *) pEntry->pFileBuffer, pEntry->fileSize);
		GetExifTopics(pEntry);
	}



	if (pEntry->topics.empty())
		treeCtrl->AppendItem(idRoot, "No EXIF Info available");

	for (size_t i = 0; i<pEntry->topics.size(); ++i)
	{
		wxTreeItemId id1 = treeCtrl->AppendItem(idRoot, pEntry->topics[i].name.c_str(), i+1);
		treeCtrl->SetItemBold( id1);
		for (size_t j = 0; j<pEntry->topics[i].properties.size(); ++j)
		{
			const ExifProperty & p = pEntry->topics[i].properties[j];
			treeCtrl->AppendItem(id1, (p.name+" : " + p.value).c_str(), 0);

		}

	}
	treeCtrl->Thaw();
}



////////////////////////////////////////////////////////////////////////////////////////////
//
//    Tags 
//
////////////////////////////////////////////////////////////////////////////////////////////

wxTreeCtrl * AlbumInfoBook::GetTagsTree()
{
  // We set the height to 65, as the real page size (set when we flip the pages)
  // can only increase the size of the control on my wxGTK2.5.5
	wxTreeCtrl * pTagsTree = new wxTreeCtrl (this, ID_TAGS_TREE, wxPoint(theme::panelLeftEdge,400), wxSize(theme::panelWidth,65), wxSUNKEN_BORDER |/* wxTR_LINES_AT_ROOT | */  wxTR_NO_LINES | wxTR_HAS_BUTTONS | wxTR_EDIT_LABELS );

	wxImageList * pIL = new wxImageList(16,16);
	pIL->Add(GET_BITMAP(16_item));
	pIL->Add(GET_BITMAP(16_core2));
	pIL->Add(GET_BITMAP(16_folder));
	pIL->Add(GET_BITMAP(16_folder_open));
	pIL->Add(GET_BITMAP(16_image));
	pTagsTree->AssignImageList(pIL);


	wxTreeItemId tagRoot = pTagsTree->AddRoot("Tags", 2, 3);
	wxTreeItemId tagPeople = pTagsTree->AppendItem(tagRoot, "People", 2, 3);
	pTagsTree->AppendItem(tagPeople, "Me", 0);

	wxTreeItemId tagPlaces = pTagsTree->AppendItem(tagRoot, "Places", 2, 3);

	wxTreeItemId tagEvents = pTagsTree->AppendItem(tagRoot, "Events", 2, 3);
 	pTagsTree->AppendItem(tagEvents, "Birthday", 0);
 	pTagsTree->AppendItem(tagEvents, "Xmas", 0);

	return pTagsTree;
}


void AlbumInfoBook::OnTagsCommand(wxCommandEvent & event)
{
	wxTreeItemId itemId = m_pTagsTree->GetSelection();
	switch (event.m_id)
	{
	case ID_TAGS_MENU_ADD_GROUP :
		{
			wxTreeItemId id = m_pTagsTree->AppendItem(itemId, "New Tag", 2, 3);
			m_pTagsTree->Expand(itemId);
			m_pTagsTree->SelectItem(id);
			break;
		}
	case ID_TAGS_MENU_ADD_TAG:
		{
			wxTreeItemId id = m_pTagsTree->AppendItem(itemId, "New Tag", 0);
			m_pTagsTree->SelectItem(id);
			m_pTagsTree->Expand(itemId);
			break;
		}
	case ID_TAGS_MENU_RENAME :
		{
			wxString newName = wxGetTextFromUser("Please enter the new Tag name");
			if (!newName.empty())
				m_pTagsTree->SetItemText(itemId, newName);
			break;
		}
	case ID_TAGS_MENU_DELETE:
		{
			int answer = wxMessageBox("Are you sure you want to Delete the tag ?", "Confirmation", wxYES_NO | wxICON_EXCLAMATION );
			if (answer==wxYES)
				m_pTagsTree->Delete(itemId);
			
			break;
		}
	default:
		BREAKPOINT;
	}
	m_pTagsTree->Refresh();
}


void AlbumInfoBook::OnTagsRightClick(wxTreeEvent &event)
{
	wxMenu menu;

	wxMenu * pAddMenu = new wxMenu;
	pAddMenu->Append(ID_TAGS_MENU_ADD_TAG, "Tag");
	pAddMenu->Append(ID_TAGS_MENU_ADD_GROUP, "Group");
	menu.Append(-1, "Add", pAddMenu);
	menu.AppendSeparator();

	menu.Append(ID_TAGS_MENU_RENAME, "Rename");
	menu.Append(ID_TAGS_MENU_DELETE, "Delete");
	menu.AppendSeparator();
	menu.Append(ID_TAGS_MENU_TOGGLE, "Toggle");
	/*m_pTagsTree->*/PopupMenu(&menu);

}

void AlbumInfoBook::BeginTagsDrag(wxTreeEvent &event)
{
	wxTreeItemId itemId = m_pTagsTree->GetSelection();
	wxTextDataObject my_data(m_pTagsTree->GetItemText(itemId));
	wxDropSource dragSource( this );
	dragSource.SetData( my_data );
	wxDragResult result = dragSource.DoDragDrop( TRUE );

}

