#include "stdafx.h"

#include "Browser.h"
#include "AlbumPrefetch.h"
#include "AlbumView.h"


AlbumPrefetch * AlbumPrefetch::Instance()
{
	static AlbumPrefetch * pInstance = new AlbumPrefetch();
	return pInstance;
}


AlbumPrefetch::AlbumPrefetch()
{
	m_pMutex = new wxMutex;
	m_pCondition = new wxCondition(*m_pMutex);
	wxThreadHelper::Create();
	GetThread()->Run();
	m_pAlbumView = 0;
	m_predict = -1;
	m_prefetching = false;
	m_exit = false;
}

AlbumPrefetch::~AlbumPrefetch()
{
	delete m_pCondition;
	delete m_pMutex;
}

void AlbumPrefetch::Stop()
{
  m_pMutex->Lock();
  m_exit = true;
  m_pMutex->Unlock();
  m_pCondition->Broadcast();
}

void AlbumPrefetch::TryPrefetch(AlbumView * pView, int i, bool goingUp)
{
	if ((m_predict!=i || m_goingUp!= goingUp || m_pAlbumView!=pView) && !m_prefetching)
	{
		m_pMutex->Lock();
		m_predict=i;
		m_pAlbumView=pView;
		m_goingUp=goingUp;
		m_pMutex->Unlock();
		m_pCondition->Signal();
	}

}

wxThread::ExitCode AlbumPrefetch::Entry()
{
	m_pMutex->Lock();
	while (1)
	{
	  m_pCondition->Wait(); 
	  if (m_exit)
	    return (0);

		m_prefetching=true;
//		long start = ::wxGetElapsedTime(false);

		size_t size = m_pAlbumView->m_entries.size();
		int i = m_predict;
		int j = m_pAlbumView->GetColumns();
		if (!m_goingUp) // if scrolling towards the bottom
		{
			while (i<signed(size) && j--)
				m_pAlbumView->LoadThumbnail( m_pAlbumView->m_entries[i++], true);
		}
		else
		{
			while (i<signed(size) && i>=0 && j--)
				m_pAlbumView->LoadThumbnail( m_pAlbumView->m_entries[i--], true);
		}

				
		m_prefetching=false;
	}
	return 0;
}


