#include "stdafx.h"

#include "AlbumRenderer.h"
#include "../images/bitmaps.h"
#include "ThemeDrawing.h"

#ifdef USE_GDIPLUS_ALBUM

#include "gdiplus.h"
using namespace Gdiplus; 


AlbumRenderer::AlbumRenderer(AlbumView * pView, wxDC & dc, int screen_width, int mn_y, int mx_y)
	: min_y(mn_y), max_y(mx_y), pAlbumView(pView)
{
	static bool gdiPlusInited = false;
	if (!gdiPlusInited)
	{
		Gdiplus::GdiplusStartupInput gdiplusStartupInput;
		ULONG_PTR           gdiplusToken;
		Gdiplus::GdiplusStartup(&gdiplusToken, &gdiplusStartupInput, NULL);
		gdiPlusInited=true;
	}

	pScreenGraphics = new Graphics((HDC) dc.GetHDC());

	// and we create a band of bitmap just for that
	pBitmap = new Bitmap(screen_width, max_y-min_y, pScreenGraphics);	

	pBitmapGraphics = new Graphics(pBitmap);
	pBitmapGraphics->Clear(Color(238,238,250));

}

AlbumRenderer::~AlbumRenderer()
{
	// and we draw the bmp by calculating back its position on screen using the canvas coordinates 
	pScreenGraphics->DrawImage(pBitmap,  Rect(0, min_y, pBitmap->GetWidth(), pBitmap->GetHeight()));

	delete pBitmap;
	delete pScreenGraphics;
	delete pBitmapGraphics;
}


void AlbumRenderer::DrawBitmap(void * pThumbnail, int x, int y)
{
	Gdiplus::CachedBitmap * pCachedBitmap = static_cast<Gdiplus::CachedBitmap *> (pThumbnail);
	pBitmapGraphics->DrawCachedBitmap( pCachedBitmap, x, y);
}

void AlbumRenderer::DeleteBitmap(void * pThumbnail)
{
	Gdiplus::CachedBitmap * pCachedBitmap = static_cast<Gdiplus::CachedBitmap *> (pThumbnail);
	delete pCachedBitmap;
}

void AlbumRenderer::DrawSelection(int x, int y, int tw, int th)
{
	Gdiplus::Pen pen1 (Color(150, 70, 70));
	Gdiplus::Pen pen2 (Color(250, 50, 50));
	Gdiplus::Pen pen3 (Color(250, 110, 110));

	pBitmapGraphics->DrawRectangle(&pen1, x-1, y-1, tw+2, th+2);
	pBitmapGraphics->DrawRectangle(&pen2, x,   y,   tw,   th);
	pBitmapGraphics->DrawRectangle(&pen3, x+1, y+1, tw-2, th-2);
}


void AlbumRenderer::DrawTag(int x, int y, int tw, int th)
{
	static Gdiplus::Bitmap * pTagBitmap = 0;

	if (!pTagBitmap)
	{
		wchar_t * ws= L"e:\\dev\\icons\\tag.gif";
		pTagBitmap = new Gdiplus::Bitmap (ws);
	}
	int xx = x+tw-pTagBitmap->GetWidth();
	int yy = y+th-pTagBitmap->GetHeight();
	pBitmapGraphics->DrawImage(pTagBitmap,xx, yy);
}

void AlbumRenderer::DrawDropShadow(int x, int y, int tw, int th) 
{
	static std::map<int, Gdiplus::Bitmap * > shadows;
	const size_t margin=8;
	const int thumb_width  = pAlbumView->thumb_width;
	const int thumb_height = pAlbumView->thumb_height;

	Gdiplus::Bitmap * & pShadowBitmap = shadows[thumb_width];

	if (!pShadowBitmap)
	{

		common::Image * pImage = pAlbumView->GetDropShadow();

		pShadowBitmap = new Gdiplus::Bitmap(thumb_width+2*margin, thumb_height+2*margin, 
										    (thumb_width+2*margin)*3, PixelFormat24bppRGB, 
											pImage->buffer);
		
	}

	int dx=3; int dy=4;
	if (tw!=thumb_width) // vertical picture. We split in two horizontally
	{
		pBitmapGraphics->DrawImage(pShadowBitmap, x-margin+dx, y-margin+dy, 
			                                0, 0, 
											margin+ tw/2, thumb_height+margin*2, UnitPixel);
		
		pBitmapGraphics->DrawImage(pShadowBitmap, x+ tw/2 +dx, y-margin +dy, 
			                                margin+thumb_width-tw/2, 0, 
											tw/2+margin, thumb_height+margin*2, UnitPixel);
	}
	else // we split in two vertically
	{
		pBitmapGraphics->DrawImage(pShadowBitmap, x-margin +dx, y-margin +dy, 
											0, 0, 
											thumb_width+margin*2, margin+th/2, UnitPixel);

		pBitmapGraphics->DrawImage(pShadowBitmap, x-margin +dx, y+th/2 +dy, 
											0, margin+thumb_height-th/2, 
											thumb_width+margin*2, th/2+margin, UnitPixel);
	}
}



void AlbumRenderer::DrawTitle(int origin) 
{
	const std::string & timeSpan = pAlbumView->m_timespan;
	const std::string & name = pAlbumView->m_name;
	const std::string & path = pAlbumView->m_path;

	SolidBrush bgBrush(Color(180, 190, 225));
	SolidBrush fgBrush(Color(4, 4, 90));

	int width=pBitmap->GetWidth(), height=pAlbumView->margin_top-40;
	pBitmapGraphics->FillRectangle(&bgBrush, 5, origin+10, width-5, height);
//	DrawShadow(pGraphics, 5, origin+2, width, height);

	Gdiplus::Pen pen (Color(112, 112, 128));

	wchar_t * ws1= new wchar_t[name.size()+1]; 
	mbstowcs(ws1, name.c_str(), name.size()*2+2);
	Font font1(L"Arial", 13, FontStyleBold);
	PointF pos1(5.0f, 20+origin);
	pBitmapGraphics->DrawString(ws1, name.size(), &font1, pos1, &fgBrush);
	delete [] ws1;



	ws1= new wchar_t[timeSpan.size()+1]; 
	mbstowcs(ws1, timeSpan.c_str(), timeSpan.size()*2+2);

	RectF layoutRect(0, 0, 1000, 50);
	RectF boundRect;
	pBitmapGraphics->MeasureString(ws1, timeSpan.size(), &font1, layoutRect, &boundRect);

	PointF pos1b(5+width-boundRect.Width, 20+origin);
	pBitmapGraphics->DrawString(ws1, timeSpan.size(), &font1, pos1b, &fgBrush);
	delete [] ws1;



	wchar_t * ws2= new wchar_t[path.size()+1]; 
	mbstowcs(ws2, path.c_str(), path.size()*2+2);
	Font font2(L"Arial", 10, FontStyleItalic);
	PointF pos2(5.0f, 60+origin);
	pBitmapGraphics->DrawString(ws2, path.size(), &font2, pos2, &fgBrush);
	delete [] ws2;
}

// static !
void AlbumRenderer::FillScreen(wxDC & dc, int screen_pos) 
{
	int min_y = screen_pos+dc.GetDeviceOrigin().y;
	int max_y = dc.GetSize().y;
	if (min_y>=max_y)
		return;

	Graphics screenGraphics((HDC) dc.GetHDC());
	Bitmap bitmap(dc.GetSize().x-24, max_y-min_y+1, &screenGraphics);	
	Graphics bitmapGraphics(&bitmap);
	bitmapGraphics.Clear(Color(238,238,250));

	screenGraphics.DrawImage(&bitmap,  Rect(0, screen_pos, bitmap.GetWidth(), bitmap.GetHeight()));
}


#else

#include <wx/mstream.h>
#include <wx/dcmemory.h>
#include <wx/image.h>

AlbumRenderer::AlbumRenderer(AlbumView * pView, wxDC & dc, int screen_width, int mn_y, int mx_y)
	: screenDc (dc),  min_y(mn_y), max_y(mx_y), pAlbumView(pView)

{

	// and we create a band of bitmap just for that
	m_pBitmap = new wxBitmap(screen_width, max_y-min_y+1);
	pBitmapDc = new wxMemoryDC;
	pBitmapDc->SelectObject(*m_pBitmap);
	pBitmapDc->SetBrush(wxBrush(wxColour(238,238,250)));
	pBitmapDc->SetPen(*wxTRANSPARENT_PEN);
	pBitmapDc->DrawRectangle(0, 0, m_pBitmap->GetWidth(), m_pBitmap->GetHeight());
}

AlbumRenderer::~AlbumRenderer()
{
	screenDc.Blit(0, min_y, m_pBitmap->GetWidth(), m_pBitmap->GetHeight(), pBitmapDc, 0, 0);
	delete pBitmapDc;
	delete m_pBitmap;
}

void AlbumRenderer::DrawBitmap(void * pThumbnail, int x, int y)
{
	wxBitmap * pBitmap = static_cast<wxBitmap *> (pThumbnail);
	pBitmapDc->DrawBitmap( *pBitmap, x, y , false);
}

void AlbumRenderer::DeleteBitmap(void * pThumbnail)
{
	wxBitmap * pBitmap = static_cast<wxBitmap *> (pThumbnail);
	delete pBitmap;
}

void AlbumRenderer::DrawSelection(int x, int y, int tw, int th)
{
  wxDC & dc = *pBitmapDc;
	dc.SetBrush(*wxTRANSPARENT_BRUSH);
	dc.SetPen(wxPen(wxColour(220, 70, 70)));


	dc.DrawRectangle(x-1, y-1, tw+2, th+2);
	dc.DrawRectangle(x,   y,   tw,   th);
	dc.DrawRectangle(x+1, y+1, tw-2, th-2);
}


void AlbumRenderer::DrawTag(int x, int y, int tw, int th)
{
	static wxBitmap * pTagBitmap = 0;

	if (!pTagBitmap)
	  {
	    wxMemoryInputStream stream(bmp_tag, sizeof bmp_tag);
	    wxImage img(stream, wxBITMAP_TYPE_GIF);
	    pTagBitmap = new wxBitmap(img);//stream, wxBITMAP_TYPE_GIF);
	  }

	int xx = x+tw-pTagBitmap->GetWidth();
	int yy = y+th-pTagBitmap->GetHeight();
	pBitmapDc->DrawBitmap(*pTagBitmap, xx, yy, true);
}

void AlbumRenderer::DrawDropShadow(int x, int y, int tw, int th) 
{

	static std::map<int, wxBitmap * > shadows;
	const size_t margin=8;
	const int thumb_width  = pAlbumView->thumb_width;
	const int thumb_height = pAlbumView->thumb_height;

	wxBitmap * & pShadowBitmap = shadows[thumb_width];
	//	static wxMemoryDC * pMemDC = 0;
	if (!pShadowBitmap)
	{
		common::Image * pImage = pAlbumView->GetDropShadow();
		wxImage img(pImage->width, pImage->height, pImage->buffer, true);
		pShadowBitmap = new wxBitmap(img);
		delete pImage;
	}
	static wxMemoryDC * pMemDC = new wxMemoryDC;;
	pMemDC->SelectObject(*pShadowBitmap);

	int dx=2; int dy=3;
	if (tw!=(signed)thumb_width) // vertical picture. We split in two horizontally
	{

		pBitmapDc->Blit(x-margin+dx, y-margin+dy, 
			    margin+ tw/2, thumb_height+margin*2,  
				pMemDC, 0, 0);
		
		pBitmapDc->Blit(x+ tw/2 +dx, y-margin +dy, 
			    tw/2+margin, thumb_height+margin*2,
				pMemDC,	margin+thumb_width-tw/2, 0);

	}
	else // we split in two vertically
	{
		pBitmapDc->Blit( x-margin +dx, y-margin +dy, 
				 thumb_width+margin*2, margin+th/2,
				 pMemDC, 0, 0);

		pBitmapDc->Blit(x-margin +dx, y+th/2 +dy, 
			    thumb_width+margin*2, th/2+margin,
			    pMemDC, 0, margin+thumb_height-th/2);

	}
	//	delete pMemDC;
}

void AlbumRenderer::DrawTitle(int origin)
{
	wxDC & dc = *pBitmapDc;

	dc.SetBrush(wxBrush(wxColour(220, 220, 240)));
	dc.SetPen(wxPen(wxColour(164, 164, 210)));

	dc.DrawRoundedRectangle (5, 5+origin, 500, 80, 4);

	static wxFont * f1 = wxFont::New(13, wxFONTFAMILY_SWISS, wxFONTFLAG_BOLD   | wxFONTFLAG_ANTIALIASED);
	static wxFont * f2 = wxFont::New(10, wxFONTFAMILY_SWISS, wxFONTFLAG_ITALIC | wxFONTFLAG_ANTIALIASED);
	dc.SetTextForeground(wxColour(4,4,90));

	dc.SetFont(*f1);
	dc.DrawText(pAlbumView->m_name.c_str(), wxPoint(9, 10+origin));

	wxCoord w,h;
	dc.GetTextExtent(pAlbumView->m_timespan.c_str(), &w, &h);
	dc.DrawText(pAlbumView->m_timespan.c_str(), wxPoint(500-w, 10+origin));


	dc.SetFont(*f2);
	dc.DrawText(pAlbumView->m_path.c_str(), wxPoint(9, 50+origin));
}




// static ! 
void AlbumRenderer::FillScreen(wxDC & dc, int screen_pos) 
{
	//TODO: put all computation in a Rect
	int min_y = screen_pos+dc.GetDeviceOrigin().y;
	int max_y = dc.GetSize().y;
	if (min_y>=max_y)
		return;
	dc.SetPen(*wxTRANSPARENT_PEN);
	dc.SetBrush(wxBrush(wxColour(238,238,250)));

	dc.DrawRectangle(wxRect(0,min_y, dc.GetSize().x-24, max_y-min_y));
}


#endif


