#include "stdafx.h"

#include "AlbumView.h"
#include "AlbumPrefetch.h"
#include "AlbumCanvas.h"
#include "AlbumRenderer.h"
#include "Browser.h"
#include "Persistence.h"
#include "ThemeDrawing.h"

#include "transform.h"
#include "gaussian.h"

#include <wx/dcclient.h>
#include <wx/image.h>
#include <wx/thread.h>

#include <sstream>
#include <iostream>

wxMutex * AlbumView::s_pThumbnailLock= new wxMutex; 

static const int thumbs_margin = 8;

extern AlbumCanvas::Activity m_activity;

AlbumView::AlbumView(AlbumCanvas * pCanvas, browser::AlbumInfo * pAlbumInfo)
		:m_pCanvas (pCanvas),  displayed(false), m_minDate(time(0)), m_maxDate(0), m_pAlbumInfo(pAlbumInfo)
{
//	m_pThumbnailLock = new wxMutex;
	thumb_width = 128;
	thumb_height = 128;
	margin_top = 120;
	margin_bottom = 40;

	grid_x = thumb_width+thumb_width/16;
	grid_y = thumb_height+thumb_height/8;

	for (size_t i =0; i<pAlbumInfo->entries.size(); ++i)	
	{
		browser::FileEntry * pFileEntry = pAlbumInfo->entries[i];
		browser::AlbumEntry * pAlbumEntry = new browser::AlbumEntry;
		pAlbumEntry->pFileEntry = pFileEntry;
		pAlbumEntry->index = i;
	
		m_entries.push_back(pAlbumEntry);   
		if (m_minDate>pFileEntry->pictureDate)
			m_minDate=pFileEntry->pictureDate;

		if (m_maxDate<pFileEntry->pictureDate)
			m_maxDate=pFileEntry->pictureDate;
	}

	m_name = pAlbumInfo->name;
	m_path = pAlbumInfo->path;
	CalculateTimeSpan();
	OnResize();
}




AlbumView::~AlbumView()
{
	// this is to prevent a destruction while the prefetch thread is active
	// TODO: we must warn the prefetch thread that we died.
	// TODO: REMOVE ALL THAT once the static lock (shared across all instances) is finished
//	s_pThumbnailLock->Lock();
//	delete m_pThumbnailLock;

}



void AlbumView::DiscardAlbum()
{
	if (!displayed)
		return;
	displayed=false;
	for (size_t i=0; i<m_entries.size(); ++i)
	{
		browser::FileEntry * pEntry = m_entries[i]->pFileEntry;
		delete pEntry->thumbnail;
		pEntry->thumbnail = 0;

		AlbumRenderer::DeleteBitmap(m_entries[i]->sysThumbnail);
		m_entries[i]->sysThumbnail = 0;
	}

}


static const char * monthName[] = { "Jan.", "Feb.", "March", "Apr.", "May", "June", "Jul.", "Aug.", "Sept.", "Oct.", "Nov", "Dec."};
static const char * dateSeparator = " - ";
void AlbumView::CalculateTimeSpan()
{
	tm * minDate =  localtime(&m_minDate);
	
	int minyear = minDate->tm_year;
	int minMonth = minDate->tm_mon;
	int minDay = minDate->tm_mday;

	tm * maxDate = localtime (&m_maxDate);

	std::stringstream result;

	if (minyear != maxDate->tm_year)
	{ // we write Month1 Year1 - Month2 Year 2
		result <<  monthName[minMonth] << " " <<  minyear+1900 << dateSeparator 
			   <<  monthName[maxDate->tm_mon] << " " <<  maxDate->tm_year+1900;
	}
	else if (minMonth != maxDate->tm_mon)
	{// This is the Apr-May 2004
		result <<  monthName[minMonth] <<  dateSeparator 
			<<  monthName[maxDate->tm_mon] << " " <<  maxDate->tm_year+1900;
	}
	else if (minDay != maxDate->tm_mday)
	{// This is the 4-6 Apr 2004
		result <<  minDay <<  dateSeparator 
			   <<  maxDate->tm_mday << " " << monthName[minDate->tm_mon] << " " << maxDate->tm_year+1900;
	}
	else
	{
		// just one day.
		result 	<<  maxDate->tm_mday << " " << monthName[minDate->tm_mon] << " " << maxDate->tm_year+1900;
	}

	m_timespan = result.str();
}

void AlbumView::SetThumbnailSize(int x)
{
	thumb_width = thumb_height = x;
	margin_top = 100+thumb_height/2;
	margin_bottom = thumb_height/2;
	grid_x = thumb_width+thumb_width/16;
	grid_y = thumb_height+thumb_height/8;
	OnResize();
	int screen_width = m_pCanvas->GetClientSize().x;
	grid_x = (screen_width-8)/(GetColumns());
}


void AlbumView::OnResize() // called by the AlbumCanvas
{
	int r = m_pCanvas->GetClientSize().x/(thumb_width+24);	
	m_columns = r ==0? 3 : r;

	m_height = margin_top + margin_bottom + (m_entries.size()+GetColumns()-1)/GetColumns()* grid_y;
}


//enum ClickTarget { ctEmpty, ctAlbumTitle, ctThumbnail, ctTag }; // TODO: ranking, album button, etc.
AlbumView::ClickTarget AlbumView::GetClickPosition(base::offset x, base::offset y, browser::AlbumEntry *&pEntry)
{
	const base::offset N = GetColumns();

	if (y<margin_top)
		return ctAlbumTitle;

	if ((y-margin_top)%grid_y>thumb_height)
		return ctEmpty;

	// left margin
	if (x<8)
		return ctEmpty;
	// margin between pictures
	if ((x-8)%grid_x>thumb_width)
		return ctEmpty;

	// right margin
	if (x-8>grid_x*N-8)
		return ctEmpty;

	size_t val = (x-8)/grid_x + N*((y-margin_top)/grid_y);

	// empty space at the bottom right of an album
	if (val >= m_entries.size())
		return ctEmpty;

	pEntry = m_entries[val];
	// Now, check that it's not in the dead-zone of the thumbnail
	if ((x-8)%grid_x<(thumb_width-pEntry->thumb_width)/2) 
		return ctEmpty;

	if ((x-8)%grid_x>(thumb_width - (thumb_width-pEntry->thumb_width)/2)) 
		return ctEmpty;

	if ((y-margin_top)%grid_y<(thumb_height - pEntry->thumb_height)/2) 
		return ctEmpty;

	if ((y-margin_top)%grid_y>(thumb_height - (thumb_height-pEntry->thumb_height)/2)) 
		return ctEmpty;

	return ctThumbnail;
}




bool AlbumView::PictureOnScreen(browser::AlbumEntry * pEntry, int shiftY, int maxY)
{
	int pos_y = shiftY + margin_top+(pEntry->index/GetColumns()) * grid_y;
	bool onScreen = ((pos_y +signed(thumb_height)+ thumbs_margin)>0) && ((pos_y-thumbs_margin)<maxY);
	if (onScreen)
		LoadThumbnail(pEntry);

	return onScreen;
}



void AlbumView::Display(wxDC & dc, int scroll_y, int mn_y, int mx_y) 
{
	int screen_width = dc.GetSize().x - 24;
	int screen_pos = this->screen_pos - scroll_y;

	// First, for added eye-candy, we shift things a bit around.
	grid_x = (screen_width-8)/(GetColumns());

	// Using the *screen coordinates*, decide what's the range to be displayed.
	// first, assume we want to draw the whole album
	int min_y = screen_pos;
	int max_y = screen_pos+this->GetHeight();

	// if there's nothing to be shown (too high or too low), exit
	if (max_y<mn_y ||min_y>mx_y)
		return;

	displayed=true;

	// then we clip this, to only the strict minimum needed: what's actually on screen
	if (min_y <mn_y) 
		min_y=mn_y;

	if (max_y> mx_y)
		max_y=mx_y;


	// We know we're scrolling down (going up) if the band to display is at the top of the screen :)
	bool goingUp = (mn_y ==0);

	AlbumRenderer renderer(this, dc, screen_width, min_y, max_y);

	Render(&renderer, screen_pos, goingUp, min_y, max_y);

}


void AlbumView::Render(AlbumRenderer * pRenderer, int screen_pos,  bool goingUp, int min_y, int max_y) 
{
	// If we can see the title, display it
	if (screen_pos<=max_y && screen_pos+(signed)margin_top>=min_y)
		pRenderer->DrawTitle (10 + screen_pos -min_y);


	size_t i =0, size = m_entries.size();
	// Now, check if *any* picture is above the max_y line.
	// If there is, find which ones
	if (screen_pos+(signed)margin_top<=max_y)
	{
		while (i<size && !PictureOnScreen(m_entries[i], screen_pos-min_y,  max_y-min_y))
			i+= GetColumns();
		
		if ((::m_activity & 16) && goingUp && i>0)
		  	 	AlbumPrefetch::Instance()->TryPrefetch(this, i-1, true);


		while (i<size && PictureOnScreen(m_entries[i], screen_pos-min_y,  max_y-min_y))
			DisplayThumbnail(pRenderer, m_entries[i++], screen_pos -min_y);
	}

	// prefetch the next images from the DB
	if ((::m_activity & 16) && !goingUp && i<size)
	  	  AlbumPrefetch::Instance()->TryPrefetch(this, i, false);  
}


void AlbumView::DisplayThumbnail(AlbumRenderer * pRenderer, const browser::AlbumEntry * pEntry, 
									  int origin) 
{
	const base::offset N = GetColumns();
	if (!pEntry->sysThumbnail)
		return;
	int pos_x =  8+(pEntry->index%N) * grid_x;
	int pos_y = margin_top+(pEntry->index/N) * grid_y;

	pos_x += (thumb_width  - pEntry->thumb_width)/2; 
	pos_y += (thumb_height - pEntry->thumb_height)/2; 
	pos_y +=origin;

	int depl = pEntry->selected ?-2:0;
	pRenderer->DrawDropShadow(pos_x+depl, pos_y+depl, pEntry->thumb_width, pEntry->thumb_height);

	pRenderer->DrawBitmap(pEntry->sysThumbnail, pos_x-depl, pos_y-depl);

	if (pEntry->pFileEntry->tagged)
		pRenderer->DrawTag(pos_x-depl, pos_y-depl, pEntry->thumb_width, pEntry->thumb_height);

	if (pEntry->selected)
		pRenderer->DrawSelection(pos_x-depl, pos_y-depl, pEntry->thumb_width, pEntry->thumb_height);

}


// static !
void AlbumView::FillScreen(wxDC & dc, int screen_pos) 
{
	AlbumRenderer::FillScreen(dc, screen_pos);
}


const int dropShadowDistX=2, dropShadowDistY=3;

common::Image * AlbumView::GetDropShadow() 
{
	const size_t margin=thumbs_margin;
	// create both image for the shadow
	size_t bufferSize = (thumb_width+2*margin)*(thumb_height+2*margin)*3;

	common::Image img(thumb_width+margin*2, thumb_height+margin*2, 
			            new base::octet[bufferSize], true);

	common::Image * shadowImage = new common::Image(thumb_width+margin*2, thumb_height+margin*2, 
						new base::octet[bufferSize], true);

	// draw a nice black rectangle, with theme::canvasColor as a background
	base::octet cRed   = theme::canvasColor.Red(), 
				cGreen = theme::canvasColor.Green(), 
				cBlue  = theme::canvasColor.Blue();

	base::octet * pBuffer = img.buffer;
	for (size_t j=0;j<thumb_height+2*margin; ++j)
		for (size_t i=0;i<thumb_width+2*margin; ++i)
		{
			if (i>margin && i<thumb_width+margin && j>margin && j<thumb_height+margin)
				pBuffer[0]=0, pBuffer[1]=0, pBuffer[2]=0;
			else
				pBuffer[base::cRed]=cRed, pBuffer[base::cGreen]=cGreen, pBuffer[base::cBlue]=cBlue;
			pBuffer+=3;

		}

	// And blur it
	transform::GaussianBlur blur;
	transform::Parameters params;
	transform::Parameter p(base::param(3.5/40.0));
	params.push_back(p);
	blur.Execute(0, img, *shadowImage, params);

	return shadowImage;
}






#ifdef USE_GDI_PLUS
void ReorientImage(int orientation, Gdiplus::Bitmap & bitmap, base::size & width, base::size & height)
{
	switch (orientation)
	{
	case 0:
	case 1:
		break;

	case 6:
		bitmap.RotateFlip(Gdiplus::Rotate90FlipNone); std::swap(width, height);
		break;
	case 8:
		bitmap.RotateFlip(Gdiplus::Rotate270FlipNone); std::swap(width, height);
		break;
	default:
		BREAKPOINT;
		break;
	}
}
#endif

void ReorientImage(int orientation, wxImage & image, base::size & width, base::size & height)
{
	switch (orientation)
	{
		// see the Win32 GDI+ version for more information
	case 6:
		image = image.Rotate90(true); std::swap(width, height);
		break;
	case 8:
		image = image.Rotate90(false); std::swap(width, height);
		break;
	default:
		break;
	}
}


static base::octet * Rotate90( bool clockwise, common::Image * pImage )
{
	const long width =  pImage->width;
	const long height = pImage->height;

	base::octet * buffer = new base::octet[width*height*3];

    unsigned char *source_data = pImage->buffer;


    for (long j = 0; j < height; j++)
    {
        for (long i = 0; i < width; i++)
        {
		    unsigned char *target_data;
            if (clockwise)
                target_data = buffer + (((i+1)*height) - j - 1)*3;
            else
                target_data = buffer + ((height*(width-1)) + j - (i*height))*3;
            memcpy( target_data, source_data, 3 );
            source_data += 3;
        }
    }
	return buffer;
}

			
void ReorientImage(browser::FileEntry * pFileEntry)
{
	if (!pFileEntry->thumbnail)
		return;

	if (pFileEntry->orientation==6 || pFileEntry->orientation==8)
	{
		base::octet * buffer = Rotate90(pFileEntry->orientation==6, pFileEntry->thumbnail); 
		std::swap(pFileEntry->thumbnail->width, pFileEntry->thumbnail->height);
		delete [] pFileEntry->thumbnail->buffer;
		pFileEntry->thumbnail->buffer = buffer;

	}
}


void GetSysThumbnail(browser::AlbumEntry * pEntry, wxDC & dc, bool threaded);


void AlbumView::LoadThumbnail(browser::AlbumEntry * pEntry, bool threaded) 
{
	// done already
	if (pEntry->sysThumbnail) 
		return;

	wxMutexLocker l(*s_pThumbnailLock);

	if (!pEntry->pFileEntry->thumbnail)
	{
		Persistence::Instance()->LoadThumbnail(pEntry->pFileEntry);
		ReorientImage(pEntry->pFileEntry);
	}

	if (!pEntry->pFileEntry->thumbnail)
	{
//		BREAKPOINT;
		return;
	}


	browser::FileEntry * f = pEntry->pFileEntry;

	base::size th = thumb_height, tw = thumb_width;

	base::size width = pEntry->pFileEntry->thumbnail->width;
	base::size height = pEntry->pFileEntry->thumbnail->height;

	if (height*tw>width*th)
		tw = width*th/height;
	else
		th = height*tw/width;

	pEntry->thumb_width = tw;
	pEntry->thumb_height = th;
	wxClientDC dc(m_pCanvas);
	GetSysThumbnail(pEntry, dc, threaded);


}


#ifdef USE_GDIPLUS_ALBUM

using namespace Gdiplus;

void GetSysThumbnail(browser::AlbumEntry * pEntry, wxDC & dc, bool)
{
	base::octet * pRawBuffer = pEntry->pFileEntry->thumbnail->buffer;
	base::octet * pBuffer = pRawBuffer;
	base::size width  = pEntry->pFileEntry->thumbnail->width;
	base::size height = pEntry->pFileEntry->thumbnail->height;
	base::size stride = width*3;

	bool correctStride = stride<768 ||stride%4;

	if (correctStride)
	{
		stride = (stride + 4) & 0xFFFFFFFC;
		if (stride<768) 
			stride = 768;

		pBuffer = new base::octet[height*stride];
		for (base::offset i=0; i<height; ++i)
			memcpy(pBuffer+i*stride, pRawBuffer+i*width*3, width*3);
	}

	Bitmap bitmap(width, height, stride, PixelFormat24bppRGB, pBuffer);


	Bitmap  cloneBitmap(pEntry->thumb_width, pEntry->thumb_height, PixelFormat24bppRGB); // PixelFormat32bppRGB 
	Graphics g(&cloneBitmap);
	g.SetInterpolationMode(InterpolationModeBilinear); 
	g.DrawImage(&bitmap, Rect(0, 0, pEntry->thumb_width, pEntry->thumb_height));

	Graphics graphics((HDC)dc.GetHDC());
	pEntry->sysThumbnail = new CachedBitmap(&cloneBitmap, & graphics);
	if (correctStride)
		delete [] pBuffer;

	// Don't need the original, as it's been cached
	delete pEntry->pFileEntry->thumbnail; 
	pEntry->pFileEntry->thumbnail = 0;
}

#else

void GetSysThumbnail(browser::AlbumEntry * pEntry, wxDC & dc, bool threaded)
{
	common::Image * pThumb = pEntry->pFileEntry->thumbnail;

    size_t tw = pThumb->width;
    size_t th = pThumb->height;

	// Do the bilinear resize if necessary
	if (tw!= pEntry->thumb_width || th!=pEntry->thumb_height)
	  {
	    base::octet * pTarget = new base::octet[pEntry->thumb_width*pEntry->thumb_height*3];  
	    common::Image * newThumbnail = new common::Image(pEntry->thumb_width, pEntry->thumb_height, pTarget, true);
	    transform::BilinearResize::Resize(*pThumb,*newThumbnail);
	    delete pThumb;
	    pEntry->pFileEntry->thumbnail = newThumbnail;
	  }
	
	// If we're in the main thread, proceed...
	if (!threaded)
	  {
	    wxImage img(pEntry->thumb_width, pEntry->thumb_height, pEntry->pFileEntry->thumbnail->buffer, true);    
	    pEntry->sysThumbnail = new wxBitmap (img);

	    delete pEntry->pFileEntry->thumbnail;
	    pEntry->pFileEntry->thumbnail = 0;
	  }
}
#endif
