#ifndef Photophoria_Albumview_h
#define Photophoria_Albumview_h

#include "Browser.h"

class AlbumRenderer;
class AlbumCanvas;

class wxDC;

class AlbumView
{
public:
	AlbumView(AlbumCanvas * pCanvas, browser::AlbumInfo * pAlbumInfo);
	~AlbumView();

	void Display(wxDC & dc, int y, int min_y, int max_y);
	void DiscardAlbum ();

	enum ClickTarget { ctEmpty, ctAlbumTitle, ctThumbnail, ctTag }; // TODO: ranking, album button, etc.
	ClickTarget GetClickPosition(base::offset x, base::offset y, browser::AlbumEntry *&pEntry);

	static void FillScreen(wxDC & dc, int screen_pos);
	void OnResize(); // called by the AlbumCanvas
	void SetThumbnailSize(int x);

	inline const browser::AlbumInfo * GetAlbumInfo() const { return m_pAlbumInfo; }
	inline base::size GetHeight() const { return m_height; }

	typedef std::vector <browser::AlbumEntry *> Entries;
	Entries m_entries;
	base::offset screen_pos;

protected:
	inline base::size GetColumns() const { 	return m_columns; }

	friend class AlbumRenderer;
	friend class AlbumPrefetch;

	void LoadThumbnail(browser::AlbumEntry * f, bool threaded=false);
//	base::size GetColumns() const;

	void Render(AlbumRenderer *, int screen_pos,  bool goingUp, int min_y, int max_y); 
	void DisplayThumbnail(AlbumRenderer *, const browser::AlbumEntry *, int origin); 
	common::Image * GetDropShadow() ;


	void CalculateTimeSpan();
	bool PictureOnScreen(browser::AlbumEntry * pEntry, int shiftY, int maxY);



	// this is locked everytime a thumbnail is created.
	// It's to prevent the prefetch thread from overriding what the display thread is doing
	static wxMutex * s_pThumbnailLock; 
	AlbumCanvas * m_pCanvas;

	const browser::AlbumInfo * const m_pAlbumInfo;
	base::offset m_height;
	base::size m_columns;
	base::size thumb_width, thumb_height;
	base::size grid_x, grid_y;
	base::size margin_top, margin_bottom;

	bool displayed;
	time_t m_minDate, m_maxDate;

	std::string m_name;
	std::string m_path;
	std::string m_timespan;
};


#endif
