#include "stdafx.h"
#include "FilterPanel.h"
#include "ThemeDrawing.h"

#include "MainFrame.h"
#include "ImageCanvas.h"
#include "ExifInfo.h"
#include "ThemeDrawing.h"

#include "../images/bitmaps.h"

#include "transform.h"
#include "gaussian.h"
#include "susan.h"

#include <wx/wx.h>

int ID_APPLY_FILTER = 130;
int ID_TRY_FILTER = 131;
int ID_CANCEL_FILTER = 132;

int ID_TREE = 140;

BEGIN_EVENT_TABLE(FilterPanel,wxPanel)
EVT_BUTTON  (ID_TRY_FILTER, FilterPanel::TryFilter) 	
  EVT_BUTTON  (ID_APPLY_FILTER, FilterPanel::ApplyFilter) 	
  EVT_BUTTON  (ID_CANCEL_FILTER, FilterPanel::CancelFilter) 	
  EVT_TREE_SEL_CHANGED(ID_TREE, FilterPanel::OnTreeClick) 
END_EVENT_TABLE()

FilterPanel::FilterPanel(wxWindow * pParent, ImageCanvas * pImageCanvas, GaugeControler * pGaugeControler)
:wxPanel(pParent, -1, wxPoint(0,0), wxSize(theme::panelWidth,-1))
{
	m_currentTransform = 0;
	m_pImageCanvas = pImageCanvas;
	m_pGaugeControler = pGaugeControler;
	m_pSliderPanel = 0;
	m_currentFilter = 0;
	m_treeCtrl = 0;

	theme::SetBackgroundColor(this);

	ConstructTree();
	CreateSliderPane();

	wxBitmapButton * p;
	p = theme::GetBitmapButton(bmp_22_try, sizeof bmp_22_try, this, ID_TRY_FILTER, wxPoint(50,170));
	p->SetToolTip("Try the Filter");
	p = theme::GetBitmapButton(bmp_22_apply, sizeof bmp_22_apply, this, ID_APPLY_FILTER, wxPoint(90,170));
	p->SetToolTip("Keep the changes");
	p = theme::GetBitmapButton(bmp_22_cancel, sizeof bmp_22_cancel, this, ID_CANCEL_FILTER, wxPoint(130,170));
	p->SetToolTip("Cancel the changes");
}

FilterPanel::~FilterPanel()
{
	for (base::offset i=0; i<m_transforms.size(); ++i)
		delete m_transforms[i];
	m_transforms.clear();
}

wxTreeCtrl * FilterPanel::ConstructTree()
{
	m_treeCtrl = new wxTreeCtrl (this, 140, wxPoint(theme::panelLeftEdge,25), wxSize(theme::panelWidth-5,140), wxTR_NO_LINES | wxSUNKEN_BORDER |wxTR_HIDE_ROOT | wxTR_HAS_BUTTONS );
	m_treeCtrl->SetIndent(10);
	wxTreeItemId idRoot = m_treeCtrl->AddRoot("Filters");
	wxTreeItemId idLight = m_treeCtrl->AppendItem(idRoot, "Light");
	wxTreeItemId idDetail = m_treeCtrl->AppendItem(idRoot, "Details");
	m_treeCtrl->SetItemBold( idLight);
	m_treeCtrl->SetItemBold( idDetail);

	AddTransform(new transform::BrightnessContrast, "Brightness / Contrast", idLight);
	AddTransform(new transform::AutoCorrect, "AutoCorrect", idLight);
	AddTransform(new transform::Exposition, "Exposition", idLight);

	AddTransform(new transform::UnsharpMask, "Enhance Details", idDetail);
	AddTransform(new transform::Susan,	"Remove Noise", idDetail);
	AddTransform(new transform::GaussianBlur, "Blur", idDetail);
	//	AddTransform(new transform::NoiseDetection, "Noise Detection", idDetail);
	// Sepia : sepia color, intensity, 

	return m_treeCtrl;
}


wxPanel * FilterPanel::CreateSliderPane()
{
	m_currentTransform= m_transforms[m_currentFilter];

	const transform::Controls & controls = m_currentTransform->GetControls();

	const int topMargin = 10;
	const int bottomMargin=10;
	int height= controls.size()*40+topMargin+bottomMargin;

	delete m_pSliderPanel;

	m_sliders.clear(); // the sliders will have been deleted by the sliderpanel

	m_pSliderPanel = new wxPanel(this, -1, wxPoint(theme::panelLeftEdge,210) ,wxSize(theme::panelWidth-5,height), wxSUNKEN_BORDER);
	theme::SetBackgroundColor(m_pSliderPanel);

	for (base::offset i=0; i<controls.size(); ++i)
	{
		const transform::Control & control = controls[i];

		/* wxStaticText * static1 = */new wxStaticText(m_pSliderPanel, -1, control.name.c_str(), wxPoint(7,topMargin+i*40), wxSize(130,20));
		wxSlider * pSlider = new wxSlider(m_pSliderPanel, -1, 20, 0, 200, wxPoint(5,topMargin+15+i*40), wxSize(180,18), wxSL_AUTOTICKS  );
		pSlider->SetTickFreq(50, 50);
		pSlider->SetValue(static_cast<int>(control.defaultValue*200));
		m_sliders.push_back(pSlider);
	}
	return m_pSliderPanel;
}


void
FilterPanel::AddTransform(transform::Transform * t, const std::string & name, wxTreeItemId id)
{
	m_transforms.push_back(t);
	t->userData = m_treeCtrl->AppendItem(id, name.c_str());
}


void FilterPanel::ExecuteTransform()
{
	wxBusyCursor wait;

	m_pGaugeControler->Enable("Filter Progress");

	common::Image img = m_pImageCanvas->GetBaseImage();
	base::octet * buffer = new base::octet[3*img.width*img.height];
	common::Image img2(img.width, img.height, buffer, true);

	transform::Parameters params;
		
	for (base::offset i=0; i<m_sliders.size(); ++i)
	{
		base::param v = m_sliders[i]->GetValue()/200.0;
		params.push_back (transform::Parameter(v));
	}

	m_currentTransform->Execute(m_pGaugeControler, img, img2, params);

	m_pImageCanvas->SetFilteredImage(img2);

	m_pGaugeControler->Disable();
} 



void FilterPanel::OnTreeClick( wxTreeEvent &event  )
{
	for (base::offset i=0; i<m_transforms.size(); ++i)
		if (m_transforms[i]->userData== (unsigned long)event.GetItem())
			m_currentFilter=i;

	CreateSliderPane();
}

void FilterPanel::TryFilter( wxCommandEvent &WXUNUSED(event)  )
{
	ExecuteTransform();
}

void FilterPanel::ApplyFilter( wxCommandEvent &WXUNUSED(event)  )
{
	m_pImageCanvas->SaveFilteredImage();
}

void FilterPanel::CancelFilter( wxCommandEvent &WXUNUSED(event)  )
{
	m_pImageCanvas->CancelFilteredImage();
}

