
#include "stdafx.h"
#include "ImageCanvas.h"
#include "ImageRenderer.h"


#include <iostream>
#include <sstream>
#include <cmath>

IMPLEMENT_DYNAMIC_CLASS(ImageCanvas, wxScrolledWindow)

BEGIN_EVENT_TABLE(ImageCanvas, wxScrolledWindow)
  EVT_ERASE_BACKGROUND(ImageCanvas::OnEraseBackGround)
  EVT_PAINT(ImageCanvas::OnPaint)
  EVT_SIZE(ImageCanvas::OnResize)
  EVT_LEFT_DOWN(ImageCanvas::Clickage)
  EVT_LEFT_UP(ImageCanvas::Clickage)
  EVT_RIGHT_DOWN(ImageCanvas::Clickage)
  EVT_RIGHT_UP(ImageCanvas::Clickage)
  EVT_MOTION(ImageCanvas::MouseMotion)
END_EVENT_TABLE()

static const int s_pictureBorder = 20;
enum BitmapChoice { CURRENT_BITMAP = 0, NEW_BITMAP = 1 };




ImageCanvas::ImageCanvas( wxWindow *parent, wxWindowID id, const wxPoint &pos, const wxSize &size )
        : wxScrolledWindow( parent, id, pos, size)
{
	m_state = NO_PICTURE;
	m_pRenderer = new ImageRenderer(this);
	m_moving=false;
	m_zoom=m_startingZoom= 1.0;

	int sx, sy;
	GetClientSize(&sx, &sy);

	SetScrollbars(1, 1, sx+1, sy+1, false);
}


double ImageCanvas::ConstrainZoom(double zoom, wxSize * pSize)
{
	assert(m_state!=NO_PICTURE);
#if (!defined(USE_GDIPLUS) && !defined(USE_GTK_PIXBUF))
		return 1.0;
#endif
	int sx, sy;
	if (pSize)
		sx = pSize->GetWidth(), sy=pSize->GetHeight();
	else
		GetClientSize(&sx, &sy);

	// POP
	sx--; sy--;
	double za = zoom*m_pRenderer->GetWidth();
	double zb = zoom*m_pRenderer->GetHeight();
	if ( (za< (sx-s_pictureBorder)) && (zb<(sy-s_pictureBorder)) )
	{
		zoom = (double)(sx-s_pictureBorder)/(m_pRenderer->GetWidth());

		double zoom2 = (double)(sy-s_pictureBorder)/(m_pRenderer->GetHeight());
		if (zoom>zoom2)
			zoom = zoom2;
	}

	return zoom;
}
void ImageCanvas::LoadPicture(const browser::FileEntry * pFileEntry)
{
	static const browser::FileEntry * pPrevious = 0;

	if (!pFileEntry || pFileEntry == pPrevious)
	{
		m_dirty = true;
		return;
	}

	pPrevious = pFileEntry;

	m_pRenderer->DeleteBitmap(CURRENT_BITMAP);
	m_pRenderer->DeleteBitmap(NEW_BITMAP);

	m_pRenderer->LoadBitmap(CURRENT_BITMAP, pFileEntry);

	m_state = NO_CHANGE;
	m_zoom = ConstrainZoom(0.01);

	int sx, sy;
	GetClientSize(&sx, &sy);

	int w = static_cast<int> (m_zoom*m_pRenderer->GetWidth());
	int h = static_cast<int> (m_zoom*m_pRenderer->GetHeight());
//POP
	if (w<sx) w=sx+1;
	if (h<sy) h=sy+1;
	SetScrollbars(1, 1, w,h , 0, 0,  true); 
/*
	wxClientDC dc(this);
	PrepareDC( dc );
	wxDCClipper(dc, GetRect());


	m_pRenderer->Paint(dc, CURRENT_BITMAP, m_zoom, true);
*/
	m_dirty=true;
}

void ImageCanvas::OnResize(wxSizeEvent & event)
{
	if(m_state == NO_PICTURE)
		return;

	m_zoom = ConstrainZoom(m_zoom);

	// TODO: write some generic code to do that ...
	int xw=static_cast<int> (m_zoom*m_pRenderer->GetWidth());
	int xh=static_cast<int> (m_zoom*m_pRenderer->GetHeight());

	int sx, sy;
	GetClientSize(&sx, &sy);

	// POP
	if (xw<sx) xw=sx+1;
	if (xh<sy) xh=sy+1;
	SetScrollbars(1, 1, xw, xh, false);

	m_dirty=true;
}



void ImageCanvas::CancelFilteredImage()
{
	if (m_state==NO_PICTURE)
		return;

	m_pRenderer->DeleteBitmap(NEW_BITMAP);
	m_state = NO_CHANGE;
	Refresh();
}
void ImageCanvas::SetFilteredImage(common::Image & image)
{
	if (m_state==NO_PICTURE)
		return;
	if (m_state==MODIFIED)
		m_pRenderer->DeleteBitmap(NEW_BITMAP);
		
	m_pRenderer->SetBitmap(NEW_BITMAP, image);
	m_state = MODIFIED;
	Refresh();
}
void ImageCanvas::SaveFilteredImage()
{
	if (m_state==NO_PICTURE)
		return;

	m_pRenderer->ShiftBitmap(CURRENT_BITMAP, NEW_BITMAP);
	m_state = APPLIED;

	Refresh();
}

common::Image ImageCanvas::GetBaseImage()
{
	return m_pRenderer->GetBaseImage(CURRENT_BITMAP);
}

ImageCanvas::~ImageCanvas()
{
	m_pRenderer->DeleteBitmap(CURRENT_BITMAP);
	m_pRenderer->DeleteBitmap(NEW_BITMAP);

	delete m_pRenderer;
}


void ImageCanvas::OnEraseBackGround(wxEraseEvent& event)
{
	if (m_state==NO_PICTURE)
		event.Skip();
}
  


void ImageCanvas::Clickage(wxMouseEvent & event)
{
	if (m_state==NO_PICTURE)
		return;

  if (event.LeftDown() || event.RightDown())
  {
      GetViewStart(&m_viewPortStartX, &m_viewPortStartY);
	  event.GetPosition(&m_clickStartX, &m_clickStartY);

	  wxSize screenSize = GetClientSize();
	  m_centerStartX= screenSize.x/2; 
	  m_centerStartY=screenSize.y/2;

	  if (screenSize.x>m_zoom*m_pRenderer->GetWidth())
		  m_viewPortStartX-=(int) (screenSize.x-m_zoom*m_pRenderer->GetWidth())/2;

	  if (screenSize.y>m_zoom*m_pRenderer->GetHeight())
		  m_viewPortStartY-=(int) (screenSize.y-m_zoom*m_pRenderer->GetHeight())/2;
			  
	  m_startingZoom=m_zoom;
	  m_moving=true;
	  m_dirty=true;
	  EnableScrolling(false, false);
//	  Refresh(false);
  }    
  else if (event.LeftUp() ||event.RightUp())
  {
	  m_moving=false;
	  m_dirty=false;
	  Refresh();
	  EnableScrolling(true, true);
  }
}

void ImageCanvas::MouseMotion(wxMouseEvent & event)
{
	if (m_state==NO_PICTURE)
		return;

	if (event.LeftIsDown() ||event.RightIsDown())
	{
		if (event.RightIsDown()) // Assume we're zooming
		{
			
			int sx, sy;
			GetClientSize(&sx, &sy);
			if (m_clickStartY>event.GetY())
				m_zoom=1 + powf(m_clickStartY-event.GetY(),3)/(2000.0*sy);
			else
				m_zoom = 1- powf(event.GetY()-m_clickStartY,1)/(1.0*sy);
			m_zoom *= m_startingZoom;

			m_zoom = ConstrainZoom(m_zoom);
			int xw=static_cast<int> (m_zoom*m_pRenderer->GetWidth());
			int xh=static_cast<int> (m_zoom*m_pRenderer->GetHeight());

			int xx = static_cast<int> ((m_viewPortStartX+sx/2.0)*(m_zoom/m_startingZoom) -m_centerStartX);
			int xy = static_cast<int> ((m_viewPortStartY+sy/2.0)*(m_zoom/m_startingZoom) -m_centerStartY);
			if (xx<0) xx=0;
			if (xy<0) xy=0;
			//Freeze();
			// POP
			if (xw<=sx) xw=sx+1;
			if (xh<=sy) xh=sy+1;
			

			//   SetVirtualSize( xw , xh);
			SetScrollbars(1, 1, xw, xh, xx, xy, true);
			//		Scroll(xx, xy);
			//		Thaw();

		}
		else // assume we're moving
		{
			int sx, sy;
			event.GetPosition(&sx, &sy);
			Scroll (m_viewPortStartX - sx + m_clickStartX, m_viewPortStartY - sy + m_clickStartY);
		}
	m_dirty=true;
	Refresh(false);
	 Update();
	}

}
       
void ImageCanvas::OnPaint( wxPaintEvent & event )
{
	// we need this here, otherwise Mr Windows gets unhappy.
	wxPaintDC dc( this );
	PrepareDC( dc );

	if (m_state == NO_PICTURE)
		return;


	int which = (m_state == MODIFIED) ? NEW_BITMAP : CURRENT_BITMAP; 
	m_pRenderer->Paint(dc, which, m_zoom, m_dirty);
	if (m_dirty && !m_moving)
		Refresh(false); // TODO: we should only repaint the damaged area.

	m_dirty  = !m_dirty;
}

