#include "stdafx.h"

#if !defined (USE_GDIPLUS) && !defined (USE_GTK_PIXBUF)

#include "ImageRenderer.h"
#include "Browser.h"
#include <wx/image.h>

void ReorientImage(int orientation, wxImage & image, base::size & width, base::size & height);

ImageRenderer::ImageRenderer(wxWindow * )
{
	bitmaps.push_back(0); 
	bitmaps.push_back(0);
}

void ImageRenderer::DeleteBitmap(unsigned int b) 
{
  delete bitmaps[b];
  bitmaps[b] = 0;
}

void ImageRenderer::ShiftBitmap(unsigned newPos, unsigned oldPos)
{
	if (bitmaps[oldPos])
	{
		delete bitmaps[newPos];
		bitmaps[newPos]=bitmaps[oldPos];
		bitmaps[oldPos]=0;
	}
}

void ImageRenderer::LoadBitmap(unsigned int pos, const browser::FileEntry * pFileEntry)
{
	wxImage img(pFileEntry->path.c_str(), wxBITMAP_TYPE_JPEG);
	base::size w,h;
	ReorientImage(pFileEntry->orientation, img, w, h);

	bitmaps[pos] = new wxBitmap(img);
}


void ImageRenderer::SetBitmap(unsigned int pos, const common::Image & image)
{
  wxImage img (image.width, image.height, image.buffer, true);
  bitmaps[pos] = new wxBitmap(img);
}


common::Image ImageRenderer::GetBaseImage(unsigned int pos) const
{
	wxImage img = bitmaps[pos]->ConvertToImage();

	size_t bitmapSize = GetWidth()*GetHeight()*3;
	base::octet * buffer= new base::octet[bitmapSize];
	memcpy(buffer, img.GetData(), bitmapSize);
	return  common::Image(GetWidth(), GetHeight(), buffer, true);
}


void ImageRenderer::Paint(wxDC & dc, unsigned pos, float zoom, bool fast) const
{
  // TODO: should assert here, really. responsibility is on the caller
	if (!bitmaps[pos])
		return;  

 	dc.DrawBitmap(*bitmaps[pos], wxPoint(0, 0),false);
}


base::size ImageRenderer::GetWidth() const
{
	return bitmaps[0]->GetWidth();
}

base::size ImageRenderer::GetHeight() const
{
	return bitmaps[0]->GetHeight();
}

#endif // GDI+ or PIXBUF ?
