#include "stdafx.h"
#include <iostream>

#ifdef USE_GTK_PIXBUF

#include "ImageRenderer.h"
#include "Browser.h"

#include <gdk/gdk.h>
#include <gtk/gtk.h>
#include <gdk/gdkx.h>
#include <gdk/gdkimage.h>

void ReorientImage(int orientation, wxImage & image, base::size & width, base::size & height);

static GdkPixbuf * GetPixbuf(int width, int height, const base::octet * buf)
{
  GdkPixbuf *pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, false,  8, width, height);
  guchar *bmp = gdk_pixbuf_get_pixels(pixbuf);
  int stride = width*3;
  int bmprowinc = gdk_pixbuf_get_rowstride(pixbuf);

  for (int y = 0; y < height; y++, bmp += bmprowinc)
      memcpy(bmp, buf + y*stride, stride);

  return pixbuf;
}


ImageRenderer::ImageRenderer(wxWindow * )
{
	bitmaps.push_back(0); 
	bitmaps.push_back(0);
}

void ImageRenderer::DeleteBitmap(unsigned int b) 
{
  if (bitmaps[b])
    gdk_pixbuf_unref(bitmaps[b]);
  bitmaps[b] = 0;
}

void ImageRenderer::ShiftBitmap(unsigned newPos, unsigned oldPos)
{
	if (bitmaps[oldPos])
	{
		gdk_pixbuf_unref(bitmaps[newPos]);
		bitmaps[newPos]=bitmaps[oldPos];
		bitmaps[oldPos]=0;
	}
}

void ImageRenderer::LoadBitmap(unsigned int pos, const browser::FileEntry * pFileEntry)
{
	wxImage img(pFileEntry->path.c_str(), wxBITMAP_TYPE_JPEG);
	base::size w,h;
	ReorientImage(pFileEntry->orientation, img, w, h);
	bitmaps[pos] = GetPixbuf(img.GetWidth(), img.GetHeight(), img.GetData());
}


void ImageRenderer::SetBitmap(unsigned int pos, const common::Image & image)
{
  bitmaps[pos] = GetPixbuf(image.width, image.height, image.buffer);
}


common::Image ImageRenderer::GetBaseImage(unsigned int pos) const
{
  GdkPixbuf *pixbuf = bitmaps[pos];
  guchar *bmp = gdk_pixbuf_get_pixels(pixbuf);
  size_t bitmapSize = GetWidth()*GetHeight()*3;
  base::octet * buffer= new base::octet[bitmapSize];
  int width = GetWidth(), height=GetHeight();
  int stride = width*3;
  int bmprowinc = gdk_pixbuf_get_rowstride(pixbuf);

  for (int y = 0; y < height; y++, bmp += bmprowinc)
      memcpy(buffer + y*stride, bmp, stride);

  return  common::Image(width, height, buffer, true);
}


void ImageRenderer::Paint(wxDC & dc, unsigned pos, float zoom, bool fast) const
{
  // TODO: should assert here, really. responsibility is on the caller
	if (!bitmaps[pos])
		return;  

	wxWindowDC & windowDC = static_cast<wxWindowDC &> (dc);

	wxSize screenSize = windowDC.m_owner->GetSize();
	//		screenSize.x-=15; screenSize.y-=15;
	int viewPortStartX, viewPortStartY;
	wxScrolledWindow * w = static_cast<wxScrolledWindow *> (windowDC.m_owner);
	w->GetViewStart(&viewPortStartX, &viewPortStartY);
	wxPoint canvasOrigin(-viewPortStartX, -viewPortStartY);


	if (screenSize.x<16 || screenSize.y<16)
	  return;
	GdkPixbuf *oldBuf = bitmaps[pos];
	GdkPixbuf * newBuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, false, 8, 
					    screenSize.x, screenSize.y);


	wxSize targetSize (static_cast<int>(zoom*GetWidth()), 
			   static_cast<int>(zoom*GetHeight()) );
	int shiftX = 0, shiftY = 0;
	bool cleared = false;
 
	int clipx=canvasOrigin.x, clipy=canvasOrigin.y;	
	if (targetSize.x<screenSize.x)
	{
	  gdk_pixbuf_fill (newBuf, 0xf0f0ffff); // Color(238,238,250)
	  cleared = true;
	  shiftX = (screenSize.x-targetSize.x)/2;
	  clipx += shiftX;
	}

	if (targetSize.y<screenSize.y)
	{
	  if (!cleared)
	    gdk_pixbuf_fill (newBuf, 0xf0f0ffff); // Color(238,238,250)
	  shiftY = (screenSize.y-targetSize.y)/2;
	  clipy+=shiftY;
	}

	gdk_pixbuf_scale(oldBuf, newBuf, 
			 shiftX, shiftY,
			 screenSize.x-shiftX*2, screenSize.y-shiftY*2,
			 clipx, clipy,
			 zoom, zoom,
			 fast ? GDK_INTERP_NEAREST : GDK_INTERP_HYPER);

	gdk_draw_pixbuf( windowDC.m_window, windowDC.m_penGC,
			 newBuf,
			 0, 0, 0, 0, -1, -1,
			 GDK_RGB_DITHER_NORMAL, 0, 0);
	gdk_pixbuf_unref(newBuf);
}


base::size ImageRenderer::GetWidth() const
{
	return gdk_pixbuf_get_width(bitmaps[0]);
}

base::size ImageRenderer::GetHeight() const
{
	return gdk_pixbuf_get_height(bitmaps[0]);
}

#endif // GDK ?
