#include "stdafx.h"

#ifdef USE_GDIPLUS

#include "windows.h"
#include "gdiplus.h"
#include "Browser.h"
#include "ImageRenderer.h"
#include <wx/image.h>

// TODO: move to proj props
#pragma comment (lib, "gdiplus.lib")

using namespace Gdiplus;


// lifted from AlbumViewWin32.cpp:
void ReorientImage(int orientation, Gdiplus::Bitmap & bitmap, base::size & width, base::size & height);



ImageRenderer::ImageRenderer(wxWindow * targetWindow)
: m_pTargetWindow (targetWindow)
{
	// TODO: I know, this looks like an ugly hack. That's because it is an ugly hack
	bitmaps.push_back(0); 
	bitmaps.push_back(0);

	Gdiplus::GdiplusStartupInput gdiplusStartupInput;
	ULONG_PTR           gdiplusToken;
	Gdiplus::GdiplusStartup(&gdiplusToken, &gdiplusStartupInput, NULL);
}

void ImageRenderer::DeleteBitmap(unsigned int b) 
{
	delete bitmaps[b];
	bitmaps[b] = 0;
}

void ImageRenderer::ShiftBitmap(unsigned newPos, unsigned oldPos)
{

	if (bitmaps[oldPos])
	{
		delete bitmaps[newPos];
		bitmaps[newPos]=bitmaps[oldPos];
		bitmaps[oldPos]=0;
	}

}

void ImageRenderer::LoadBitmap(unsigned int pos, const browser::FileEntry * pFileEntry)
{
#if 0 //FORCE_GDIP_JPEG_LOADER
	wchar_t * ws= new wchar_t[pFileEntry->path.size()+1];
	mbstowcs(ws, pFileEntry->path.c_str(), pFileEntry->path.size()*2+2);
	bitmaps[pos] = new Gdiplus::Bitmap ((WCHAR*)ws);
	delete [] ws;
#else
	// If we use the MMX&SSE2 decoder, the wxWidgets jpeg lib is faster.
	// But we then have to make sure the gdi+ bitmap gets created from a Window 
	// Graphics otherwise, for some dumb reason, the display isn't as fast 
	// (regardless of the PixelFormat).
	wxImage img(pFileEntry->path.c_str(), wxBITMAP_TYPE_JPEG);
	common::Image im (img.GetWidth(), img.GetHeight(), img.GetData(), false);
	SetBitmap(pos, im);
#endif

	base::size x,y;
	ReorientImage(pFileEntry->orientation, *bitmaps[pos], x, y);

}

void ImageRenderer::SetBitmap(unsigned int pos, const common::Image & image)
{
//	bitmaps[pos] = new Bitmap(image.width, image.height, image.width*3,  PixelFormat24bppRGB , image.buffer);
	base::octet * newBuffer = new base::octet[image.width*image.height*4];
	base::octet * p=newBuffer, *p2=image.buffer;
	for (int i=image.width*image.height; i; --i)
	{
		p[0]=p2[2];
		p[1]=p2[1];
		p[2]=p2[0];
	//	p[3]=0;
		p+=3; p2+=3;
	}

	Graphics * pScreenGraphics = new Graphics((HWND) m_pTargetWindow->GetHWND(), true);

	// and we create a band of bitmap just for that
	//pBitmap = new Bitmap(screen_width, max_y-min_y, pScreenGraphics);	

	bitmaps[pos] = new Bitmap(image.width, image.height, pScreenGraphics);
	Rect rect(0, 0, image.width, image.height);
	BitmapData bitmapData;
	bitmapData.Height = image.height;
	bitmapData.Width = image.width;
	bitmapData.PixelFormat = PixelFormat24bppRGB ;
	bitmapData.Scan0 = newBuffer;
	bitmapData.Stride = image.width*3;
	bitmaps[pos]->LockBits(&rect,  ImageLockModeWrite | ImageLockModeUserInputBuf, PixelFormat24bppRGB , &bitmapData);
	bitmaps[pos]->UnlockBits(&bitmapData);
	delete [] newBuffer;
}

common::Image ImageRenderer::GetBaseImage(unsigned int pos) const
{
	size_t myStride = GetWidth()*3;

	Rect rect(0, 0, GetWidth(), GetHeight());
	BitmapData bitmapData;
	bitmapData.Height = GetHeight();
	bitmapData.Width = GetWidth();
	bitmapData.PixelFormat = PixelFormat24bppRGB;
	bitmapData.Scan0 = 0;
	bitmapData.Stride = myStride;
	bitmaps[pos]->LockBits(&rect,  ImageLockModeRead, PixelFormat24bppRGB, &bitmapData);

	base::octet * buffer = new base::octet [myStride*bitmapData.Height];
	base::octet * pDst = buffer;

	const base::octet * pSrc = static_cast<const base::octet *> (bitmapData.Scan0);
	for (size_t i=0; i<bitmapData.Height; ++i)
	{
		for (size_t j=bitmapData.Width ; j; --j)
		{
			pDst[0]=pSrc[2];
			pDst[1]=pSrc[1];
			pDst[2]=pSrc[0];
			pDst+=3; pSrc+=3;
		}
//		memcpy(pDst, pSrc, myStride);
		pSrc+=bitmapData.Stride-myStride;
	}

	bitmaps[pos]->UnlockBits(&bitmapData);

	return  common::Image(GetWidth(), GetHeight(), buffer, true);

}

void ImageRenderer::Paint(wxDC & dc, unsigned pos, float zoom, bool fast) const
{
	if (!bitmaps[pos])
		return;  // TODO: should assert here, really. responsibility is on the caller

	Graphics graphics((HDC)dc.GetHDC());
//	ImageAttributes imageAtt;

	if (fast)
		graphics.SetInterpolationMode(InterpolationModeNearestNeighbor);
	else
		graphics.SetInterpolationMode(InterpolationModeHighQualityBicubic);//InterpolationModeHighQualityBicubic);


	INT iWidth = bitmaps[pos]->GetWidth();
	INT iHeight = bitmaps[pos]->GetHeight();
	wxSize screenSize = dc.GetWindow()->GetSize(); // POP
	
	Rect targetRect (0.0, 0.0, zoom*iWidth,  zoom*iHeight);
	SolidBrush b(Color(247,247,255));

		int screenHeight = screenSize.y;
		int screenWidth = screenSize.x;
	if (targetRect.Width<screenSize.x)
	{
		if (screenHeight<targetRect.Height)
			screenHeight=targetRect.Height;

		targetRect.X += screenSize.x/2-targetRect.Width/2;
	}
	if (targetRect.Height<screenSize.y)
	{
		if (screenWidth<targetRect.Width)
			screenWidth=targetRect.Width;
		targetRect.Y += screenSize.y/2-targetRect.Height/2;
	}

	graphics.DrawImage(	bitmaps[pos],  targetRect, 0, 0, iWidth, iHeight, UnitPixel, 0);
	if (targetRect.X)
	{
		graphics.FillRectangle(&b, 0,0, targetRect.X, screenHeight); // left
		graphics.FillRectangle(&b, targetRect.X+targetRect.Width,0, screenSize.x-targetRect.X, screenHeight); // right
	}
	if (targetRect.Y)
	{
		graphics.FillRectangle(&b, 0, 0 , screenWidth, targetRect.Y); // top
		graphics.FillRectangle(&b, 0, targetRect.Y+targetRect.Height, screenWidth, screenSize.y-targetRect.Y+1); // bottom
	}
}

base::size ImageRenderer::GetWidth() const
{
	return bitmaps[0]->GetWidth();
}
base::size ImageRenderer::GetHeight() const
{
	return bitmaps[0]->GetHeight();
}

#endif // GDI+ ?
