#include "stdafx.h"
#include "MainFrame.h"
#include "Photophoria.h"
#include "AlbumCanvas.h"
#include "ImageCanvas.h"
#include "FilterPanel.h"
#include "ThemeDrawing.h"
#include "Persistence.h"
#include "StatusBar.h"
#include "AlbumInfoBook.h"
#include "AlbumController.h"

#include <wx/image.h>

#include "../images/bitmaps.h"

#include <sstream>

const int ID_BTN_QUIT  = 100;
const int ID_BTN_ALBUM = 101;
const int ID_BTN_IMAGE = 102;
const int ID_BTN_PREVIOUS = 121;
const int ID_BTN_NEXT = 122;


BEGIN_EVENT_TABLE(MyFrame,wxFrame)
  EVT_BUTTON  (ID_BTN_ALBUM, MyFrame::ShowPictureEvent) 	
  EVT_BUTTON  (ID_BTN_IMAGE, MyFrame::ShowPictureEvent) 	 	

  EVT_BUTTON  (ID_BTN_PREVIOUS, MyFrame::NextPicture)
  EVT_BUTTON  (ID_BTN_NEXT, MyFrame::NextPicture) 	
END_EVENT_TABLE()




MyFrame::MyFrame()
: wxFrame( (wxFrame *)NULL, -1, _T("Photophoria"),
		  wxDefaultPosition, wxDefaultSize, wxCLIP_CHILDREN | wxNO_FULL_REPAINT_ON_RESIZE| wxDEFAULT_FRAME_STYLE)
{
    m_pStatusBar = new StatusBar(this);
	SetStatusBar(m_pStatusBar);
	currentMode = ModeAlbum;
	

	m_pImagePanel = 0;
	m_pAlbumPanel = 0;

	m_albumCanvas = new AlbumCanvas( this, m_pStatusBar,  -1, wxPoint(0,0), wxSize(600,400) );  
	m_albumCanvas->SetBackgroundColour(theme::canvasColor);
	m_imageCanvas  = new ImageCanvas  ( this, -1, wxPoint(0,0), wxSize(1,1) );
#ifndef __WXGTK__
	m_imageCanvas->SetBackgroundColour(theme::canvasColor);
#endif

	m_pAlbumController = new AlbumController(this, m_albumCanvas);
	m_albumCanvas->SetController(m_pAlbumController);

	// Top-left Panel
	wxPanel * switcherPanel = new wxPanel(this, -1, wxPoint(0,5), wxSize(theme::panelWidth,55),wxNO_BORDER); // unused
	theme::SetBackgroundColor(switcherPanel);
	m_pBmpAlbum = theme::GetBitmapButton(bmp_browser, sizeof bmp_browser, switcherPanel , ID_BTN_ALBUM, wxPoint (15, 2));
	m_pBmpAlbum->SetToolTip("Switch to Album View");
	m_pBmpAlbum->Disable();
	m_pBmpImage = theme::GetBitmapButton(bmp_image, sizeof bmp_image, switcherPanel , ID_BTN_IMAGE, wxPoint (75, 2));
	m_pBmpImage->SetToolTip("Swith to Image View");
	m_pBmpImage->Disable();
	currentMode = ModeAlbum;



	// Button Panel
	wxPanel * buttonPanel = new wxPanel(this, -1, wxPoint(30,5), wxSize(150,55), wxCLIP_CHILDREN | wxNO_FULL_REPAINT_ON_RESIZE);
	theme::SetBackgroundColor(buttonPanel);
	buttonPanel->SetMinSize(wxSize(150,70));
	const unsigned char * names2[] = { bmp_back, bmp_forward };
	const char * tips2[] = { "Previous Picture", "Next Picture" };
	size_t sizes2[] = {sizeof bmp_back, sizeof bmp_forward};
	for (unsigned i=0; i <sizeof (names2)/ sizeof(names2[0]); ++i)
	{
		wxPoint pos(70+i*70+(i/4)*70, 2);
		wxBitmapButton * pBmp = theme::GetBitmapButton(names2[i], sizes2[i], buttonPanel, ID_BTN_PREVIOUS+i, pos);
		pBmp->SetToolTip(tips2[i]);
		pBmp->Disable();
	}
	

	// Left Panel (starts with the Album Panel only)
	wxPanel * pLeftPanel = new wxPanel(this, -1, wxDefaultPosition, wxDefaultSize );
	theme::SetBackgroundColor(pLeftPanel);
	
	m_pAlbumPanel = m_pAlbumController->GetAlbumPanel(pLeftPanel);
	m_pLeftSizer = new wxBoxSizer(wxVERTICAL);;
	m_pLeftSizer->Add(m_pAlbumPanel, 1, wxGROW, 0);
	pLeftPanel->SetSizer(m_pLeftSizer);
	m_pLeftSizer->Layout();


	// Top level Sizer 
	m_topsizer = new wxFlexGridSizer(2,2,0,0);
	m_topsizer->Add(switcherPanel,  1, wxGROW , 0);
	m_topsizer->Add(buttonPanel, 1, wxGROW , 0);

	m_topsizer->Add(pLeftPanel, 1, wxGROW  , 0); //  wxALIGN_CENTRE | wxALIGN_CENTER_VERTICAL | wxALL
	m_topsizer->Add(m_albumCanvas, 1, wxGROW , 0);
	SetSizer( m_topsizer );      // use the sizer for layout

	m_topsizer->AddGrowableRow(1);
	m_topsizer->AddGrowableCol(1);
	m_topsizer->SetSizeHints( this );   // set size hints to honour minimum size
	m_topsizer->Layout();

	
	m_pAlbumController->FillAlbumList();
	m_albumCanvas->SetAlbums(Persistence::Instance()->GetAlbums(Persistence::FOLDER));
	m_pAlbumController->FinishLoad();

}

MyFrame::~MyFrame()
{
	delete m_albumCanvas;
	delete m_imageCanvas;
}

void MyFrame::OnQuit( wxCommandEvent &WXUNUSED(event) )
{
  Close( TRUE );
}


void MyFrame::ShowPictureEvent( wxCommandEvent & event )
{
	if (event.GetId()==ID_BTN_ALBUM)
		ShowAlbum();
	else
		ShowPicture();
}

/*
void AlbumController::AddAlbum(browser::AlbumInfo * pInfo)
{
	m_pAlbumController->AddAlbum(pInfo);
}
*/

void MyFrame::NextPicture(wxCommandEvent &event)
{
	if (!m_imageCanvas || !m_albumCanvas->GetCurrentFile())
		return;

	m_albumCanvas->NextPicture(event.GetId() == ID_BTN_PREVIOUS);

	m_imageCanvas->LoadPicture(m_albumCanvas->GetCurrentFile());
	m_imageCanvas->Refresh(false);
	m_imageCanvas->Update();
}

void SwitchIt (wxPanel * newPanel, wxPanel * oldPanel, wxSizer * pSizer)
{
	oldPanel->Show(false);
	pSizer->Detach(oldPanel);
	pSizer->Add(newPanel, 1, wxGROW, 0);
//	pSizer->Layout();
	newPanel->Show(true);
	newPanel->Refresh(true);
	newPanel->Update();
}


void MyFrame::SwapMainWindow(wxWindow * newWindow, wxWindow * oldWindow)
{
	Freeze();
	oldWindow->Show(false);
	m_topsizer->Detach(oldWindow);
	m_topsizer->Add(newWindow, 1, wxGROW,0);
	m_topsizer->Layout(); 
	newWindow->Show(true);
	Thaw();
}



void MyFrame::ShowPicture( )
{
	if (currentMode == ModePicture || !m_imageCanvas)
		return;

	wxBusyCursor wait;
	if (!m_pImagePanel)
		m_pImagePanel = new FilterPanel(m_pAlbumPanel->GetParent(), m_imageCanvas, m_pStatusBar);

	currentMode = ModePicture;
	m_pBmpAlbum->Enable();
	m_pBmpImage->Disable();

	SwitchIt (m_pImagePanel, m_pAlbumPanel, m_pLeftSizer);
	m_imageCanvas->LoadPicture(m_albumCanvas->GetCurrentFile());
	SwapMainWindow(m_imageCanvas, m_albumCanvas);
	Refresh();

	FindWindow(ID_BTN_PREVIOUS)->Enable();
	FindWindow(ID_BTN_PREVIOUS+1)->Enable();
}



void MyFrame::ShowAlbum( )
{
	if (currentMode == ModeAlbum)
		return;
	currentMode = ModeAlbum;
	m_pBmpAlbum->Disable();
	m_pBmpImage->Enable();

	SwitchIt (m_pAlbumPanel, m_pImagePanel, m_pLeftSizer);
	SwapMainWindow(m_albumCanvas, m_imageCanvas);

	FindWindow(ID_BTN_PREVIOUS)->Disable();
	FindWindow(ID_BTN_PREVIOUS+1)->Disable();
	Refresh();
}


void MyFrame::UpdateSelection()
{
	// Enable the "Show Picture" Button.
	if (m_albumCanvas->GetCurrentFile() || !m_albumCanvas->GetSelection().empty())
			m_pBmpImage->Enable();
	else
			m_pBmpImage->Disable();
	
	m_pAlbumController->UpdateSelection();
}
