#ifndef Photophoria_Persistence_h
#define Photophoria_Persistence_h

#include "Browser.h"

struct sqlite3;
struct sqlite3_stmt;

class Persistence
{
public:

	class Transaction
	{
	public:
		Transaction();
		~Transaction();
	};

	static Persistence * Instance();

	~Persistence ();

	void Persistence::LoadThumbnail(browser::FileEntry * pEntry);
	bool Persistence::HasThumbnail(browser::FileEntry * pEntry);
	void Persistence::SaveThumbnail(browser::FileEntry * pEntry);

	void LoadAlbums();
	void LoadFileEntries(browser::AlbumInfo * pAlbum);
	void SaveAlbum(browser::AlbumInfo * pAlbum);
	void SaveFileEntry(browser::FileEntry * pFileEntry);

	bool AlbumExists(browser::AlbumInfo * pAlbum);
	browser::AlbumInfo::Id Persistence::GetUniqueId();

	// TODO: create an AlbumLibrary class, mix of Browser and Persistence
	typedef std::map<int, browser::AlbumInfo *> DateAlbums;
	typedef enum { FOLDER, CALENDAR} Ordering;
	const std::vector<browser::AlbumInfo *> & GetAlbums(Ordering order);
	const DateAlbums & GetMonths();

protected:
	const std::vector<browser::AlbumInfo *> & GetCalendarAlbums();
	const std::vector<browser::AlbumInfo *> & GetAllAlbums();

	Persistence (); // singleton init. Loads the DB
	void StartDb();
	void RunSql(const std::string & s, bool ignoreErrors = false);
	sqlite3_stmt * PrepareStatement (const std::string & s);

	sqlite3 * m_pDb;
	std::vector <browser::AlbumInfo *> m_albums;
	browser::AlbumInfo::Id m_currentId;
};

#endif
