#include <stdafx.h>
#include "ThemeDrawing.h"

#include <wx/bitmap.h>
#include <wx/image.h>
#include <wx/mstream.h>
#include <wx/dcmemory.h>
#include <wx/bmpbuttn.h>

using namespace theme;

wxColour theme::bgColor (247,247,255); 
wxColour theme::canvasColor (238,238,250);

void theme::SetBackgroundColor(wxWindow * p)
{
#ifndef __WXGTK__
  p->SetBackgroundColour(theme::bgColor);
#endif
}

static void getDropShadow(const wxImage & img, wxBitmap & upBmp, wxBitmap & normalBmp, wxBitmap & greyBmp)
{
	wxBitmap iconBmp(img);

	memset(img.GetData(), 160, iconBmp.GetWidth()*3*iconBmp.GetHeight());
	unsigned char * alpha = img.GetAlpha();
	unsigned long stride = img.GetWidth();

	if (alpha)
	{
		for (int y=1; y<(iconBmp.GetHeight()-1); ++y)
			for (size_t x=1; x<stride-1; ++x)
			{
				size_t pos = y*stride+x;
				alpha[pos] = (alpha[pos-1]+alpha[pos+1]+alpha[pos-stride] + alpha[pos+stride])/4;
			}
		for (int y=1; y<(iconBmp.GetHeight()-1); ++y)
			for (size_t x=1; x<stride-1; ++x)
			{
				size_t pos = y*stride+x;
				alpha[pos] = (alpha[pos-1]+alpha[pos+1]+alpha[pos-stride] + alpha[pos+stride])/4;
			}
	}
	wxBitmap shadowBmp (img);

	wxMemoryDC memDc;
	// On GTK, we need to set the brush after each SelectObject()
	wxBrush brush(wxColor(222,212,202)); //*wxTRANSPARENT_BRUSH);// (bgColor);
	wxPen pen (bgColor);


	wxPoint dropShadowPos(4,4);
	wxPoint originPos(2,2); // To please GTK
	unsigned newWidth =  iconBmp.GetWidth() + dropShadowPos.x+2;
	unsigned newHeight = iconBmp.GetHeight()+ dropShadowPos.y+2;

	wxBitmap targetBmp(newWidth, newHeight);
	memDc.SelectObject(targetBmp);
	memDc.SetBrush(brush);
	memDc.SetPen(pen);

	memDc.DrawRectangle(0,0,newWidth, newHeight);
	memDc.DrawBitmap(shadowBmp, dropShadowPos, TRUE);
	memDc.DrawBitmap(iconBmp,originPos, TRUE);
	upBmp = targetBmp;

	wxBitmap targetBmp2(newWidth, newHeight);
	memDc.SelectObject(targetBmp2);
	memDc.SetBrush(brush);
	memDc.SetPen(pen);

	memDc.DrawRectangle(0,0,newWidth, newHeight);
	memDc.DrawBitmap(iconBmp,dropShadowPos, TRUE);
	normalBmp = targetBmp2;

	wxBitmap targetBmp3(newWidth, newHeight);
	memDc.SelectObject(targetBmp3);
	memDc.SetBrush(brush);
	memDc.SetPen(pen);

	memDc.DrawRectangle(0,0,newWidth, newHeight);
	memDc.DrawBitmap(shadowBmp,dropShadowPos, TRUE);
	greyBmp = targetBmp3;
	

}

wxBitmap theme::GetBitmap(const unsigned char * buffer, size_t bufferSize)
{
	wxMemoryInputStream stream(buffer, bufferSize);
	
    wxImage img(stream, wxBITMAP_TYPE_PNG);
	wxBitmap bit(img);
	return  bit;
}

wxBitmapButton *  theme::GetBitmapButton(const unsigned char * buffer, size_t bufferSize, wxWindow * parent, unsigned id, wxPoint pos )
{
	wxMemoryInputStream stream(buffer, bufferSize);
	
	wxImage img(stream, wxBITMAP_TYPE_PNG);

	wxBitmap bitSelected, bitNormal, bitDisabled;
	getDropShadow(img, bitNormal, bitSelected, bitDisabled); 

	wxBitmapButton * pButton = new wxBitmapButton(parent, id, bitNormal, pos, wxDefaultSize, wxBORDER_NONE);
	//	pButton->SetBackgroundColour(bgColor);
	pButton->SetBitmapSelected(bitSelected);
	pButton->SetBitmapDisabled(bitDisabled);
	return pButton;
}

