#ifndef Photophoria_common_h
#define Photophoria_common_h

#include <string>
#include <vector>
#include <set>
#include <map>
#include <algorithm>

/*
#ifndef max
#define max(a,b) (((a)>(b)) ? (a) : (b))
#endif

#ifndef min
#define min(a,b) (((a)<(b)) ? (a) : (b))
#endif
*/
namespace base
{
    typedef size_t 			size;  // width, height and offsets
    typedef size_t 			offset;// offsets over dimensions
    typedef unsigned char 	octet;
    typedef float			param; // parameters passed to the transforms

	// We go and define some platform-specific things here.
#if defined(_MSC_VER)
	#define USE_GDIPLUS 1
	#define  BREAKPOINT  _asm int 3 
#else
	#define BREAKPOINT
#endif

#if defined(__WXGTK__)
   #define USE_GTK_PIXBUF 1
#endif

#if USE_GDIPLUS
	static const int cRed = 2;
	static const int cGreen = 1;
	static const int cBlue = 0;
#else
	static const int cRed = 0;
	static const int cGreen = 1;
	static const int cBlue = 2;
#endif
}


class GaugeControler
{
public:
	virtual void Enable(const std:: string & text) = 0;
	virtual void SetValue(int value) = 0;
	virtual void Disable() = 0;
};



namespace common
{
	// Some Application-wide functions. The behaviour is not guaranteed on these 2.
    void Debug(const std::string & s);
	void Yield();

	class Exception
	{
	public:
		Exception (const std::string &s ) : error(s){}
		std::string error;
	};

	class FileFormatException : public Exception
	{
	public:
		FileFormatException (const std::string &s ) : Exception(s){}
	};

    struct Image
    {
	   Image(base::size w, base::size h, base::octet * b, bool o)
        : width(w), height(h), buffer(b), owned(o)
         {
             //TODO: pixelStep isn't always 3
			 pixelStep=3;
			 scanline = width*pixelStep;
         }
		 ~Image()
		 {
			 if (owned)
				 delete [] buffer;
		 }

		 inline base::size GetBufferSize() { return scanline*height; }

       base::size width;
       base::size height;
       base::octet * buffer;
       bool owned;
      
       base::size scanline;  // distance between 2 lines
	   base::size pixelStep; // distance between 2 pixels
	private:
    };
}

#endif


