#ifndef Photophoria_gaussian_h
#define Photophoria_gaussian_h

#include "transform.h"

namespace transform
{
    //////////////////////////////////////////////:
    // Classic Gaussian Blur
    //////////////////////////////////////////////:

    class GaussianBlur : public Transform
    {
    public:
        virtual const Controls & GetControls() const
        {
            static Controls controls;
            if (controls.empty())
            {
				controls.push_back(Control("Size", Control::FLOAT_SLIDER, 0.25f));
            }
            return controls;    
        };
        
        virtual bool Execute(GaugeControler * pGaugeControler, 
							const common::Image & in, common::Image & out, 
	       					 const transform::Parameters & params)
        {
			base::param n = params[0].value*40.0;
            DoGaussian(pGaugeControler, in, out, n);
            return true; // cannot fail
        }    

        void DoGaussian(GaugeControler * pGaugeControler, const common::Image & image, common::Image & image2, base::param n);
		void CreateConvolutionVector (base::param radius);
        void ConvoluteDimension(GaugeControler * pGaugeControler, const common::Image & image, common::Image & image2, bool bHorizontal);

	protected:
		typedef int gaussianParam;
		gaussianParam m_denominator;
		gaussianParam * m_convolutionVector; //TODO: mem leak
		int m_dim;

    };    



    //////////////////////////////////////////////:
    // Unsharp Mask. Inherits from GaussianBlur
    //////////////////////////////////////////////:

    class UnsharpMask : public GaussianBlur
    {
    public:
        virtual const Controls & GetControls() const
        {
            static Controls controls;
            if (controls.empty())
            {
				controls.push_back(Control("Size", Control::FLOAT_SLIDER, 0.25f));
				controls.push_back(Control("Epsilon", Control::FLOAT_SLIDER, 0.25f));
            }
            return controls;    
        };
        
        virtual bool Execute(GaugeControler * pGaugeControler, 
							 const common::Image & in, common::Image & out, 
	       					 const transform::Parameters & params)
        {
			base::param n = params[0].value*40;
			base::param e = params[1].value;
            DoGaussian(pGaugeControler, in, out, n);
			DoUnsharpMask(in, out, e);
            return true; // cannot fail
        }    
	protected:
		void DoUnsharpMask (const common::Image & image, common::Image & image2, base::param a);
    };    


	//////////////////////////////////////////////:
	// Shadows/Highlights. Inherits from GaussianBlur
	//////////////////////////////////////////////:
	class Exposition : public GaussianBlur
	{
	public:
		virtual const Controls & GetControls() const
		{
			static Controls controls;
			if (controls.empty())
			{
				controls.push_back(Control("Highlights", Control::FLOAT_SLIDER, 0.25f));
				controls.push_back(Control("Shadows", Control::FLOAT_SLIDER, 0.0f));
				controls.push_back(Control("Correction", Control::FLOAT_SLIDER, 0.25f));
			}
			return controls;    
		};

		virtual bool Execute(GaugeControler * pGaugeControler, 
			const common::Image & in, common::Image & out, 
			const transform::Parameters & params)
		{
			base::param n = 30; // Size is fixed at 30 pixels.
			DoGaussian(pGaugeControler, in, out, n);
			DoHighlightsShadows(in, out,  params[1].value,  1-params[0].value,  
									1.0+4*params[2].value, 1.0+4*params[2].value);
			return true; // cannot fail
		}    
	protected:
		void DoHighlightsShadows (const common::Image & image, common::Image & image2,
			base::param shadow_cutoff, base::param highlight_cutoff, base::param gamma1, base::param gamma2);
	};    


} 
   
#endif

