#include "stdafx.h"
#include "common.h"
#include "susan.h"

#include <cmath>
#include <sstream>

using namespace transform;

typedef  unsigned char uchar;

static unsigned char * setup_brightness_lut(float bt, int form)
{
  static unsigned char * bp=new unsigned char[516] + 258;

  for(int k=-256;k<257;k++)
  {
	float temp=powf(k/bt,2);
    bp[k]= static_cast<unsigned char> (250.0*exp(-temp));
  }
  return bp;
}



static const unsigned char * setup_distance_lut(float dt, int & mask_size)
{
	mask_size = ((int)(1.5 * dt)) + 1;

	int n_max = (mask_size*2) + 1;
	

	unsigned char * dp = new unsigned char[n_max*n_max];
	uchar * dpt = dp;

	for(int i=-mask_size; i<=mask_size; i++)
		for(int j=-mask_size; j<=mask_size; j++)
		{
			float c = -(i*i + j*j);
			float val = exp( c /(dt*dt));
			*dpt++ = (unsigned char)(250.0 * val);
		}

	return dp;
}

static uchar median(uchar * in, int i, int j, int x_size)
{
int p[8],k,l;

  p[0]=in[(i-1)*x_size+j-1];
  p[1]=in[(i-1)*x_size+j  ];
  p[2]=in[(i-1)*x_size+j+1];
  p[3]=in[(i  )*x_size+j-1];
  p[4]=in[(i  )*x_size+j+1];
  p[5]=in[(i+1)*x_size+j-1];
  p[6]=in[(i+1)*x_size+j  ];
  p[7]=in[(i+1)*x_size+j+1];

  for(k=0; k<7; k++)
    for(l=0; l<(7-k); l++)
      if (p[l]>p[l+1])
      {
        int tmp=p[l]; p[l]=p[l+1]; p[l+1]=tmp;
      }

  return( (p[3]+p[4]) / 2 );
}


/* this enlarges "in" so that borders can be dealt with easily */
static void enlarge(uchar * in, uchar * tmp_image, int & x_size, int & y_size,  int border)
{
int   i, j;

  for(i=0; i<y_size; i++)   /* copy in into tmp_image */
    memcpy(tmp_image+(i+border)*(x_size+2*border)+border, in+i* x_size, x_size);

  for(i=0; i<border; i++) /* copy top and bottom rows; invert as many as necessary */
  {
    memcpy(tmp_image+(border-1-i)*(x_size+2*border)+border,in+i* x_size,x_size);
    memcpy(tmp_image+(y_size+border+i)*(x_size+2*border)+border,in+(y_size-i-1)* x_size,x_size);
  }

  for(i=0; i<border; i++) /* copy left and right columns */
    for(j=0; j<y_size+2*border; j++)
    {
      tmp_image[j*(x_size+2*border)+border-1-i]=tmp_image[j*(x_size+2*border)+border+i];
      tmp_image[j*(x_size+2*border)+ x_size+border+i]=tmp_image[j*(x_size+2*border)+ x_size+border-1-i];
    }

  x_size+=2*border;  /* alter image size */
  y_size+=2*border;
}


static void susan_smoothing(GaugeControler * pGaugeControler, int plane, uchar * out, uchar * in, int x_size, int y_size, 
							int mask_size, const uchar * bp, const uchar * dp)
{
	int n_max = (mask_size*2) + 1;

	for (int i=mask_size;i<y_size-mask_size;i++)
	{
		pGaugeControler->SetValue(plane*32+2+(32*i/(y_size-2*mask_size)));
		for (int j=mask_size;j<x_size-mask_size;j++)
		{
			int area = 0;
			float  total = 0;
			const unsigned char * dpt = dp;
			const unsigned char * ip = in + ((i-mask_size)*x_size) + j - mask_size;
			const unsigned char centre = in[i*x_size+j];
			const unsigned char * cp = bp + centre;
			for(int y=n_max; y; y--)
			{
				for(int x=n_max; x; x--)
				{
					int brightness = *ip++;
					unsigned int tmp = *dpt++ * *(cp-brightness);
					area += tmp;
					total += tmp * brightness;
				}
				ip += x_size - n_max;
			}
			int tmp = area-250*250;
			if (tmp==0)
				*out=median(in,i,j,x_size);
			else
				*out=static_cast<base::octet>((total-(centre*250*250))/tmp);
			out+=3;
		}
	}
}




void Susan::DoSusan(GaugeControler * pGaugeControler, const common::Image & image, common::Image & image2, 
					float dt, float bt1, float bt2)
{
	int  mask_size;
	const unsigned char * dp = setup_distance_lut(dt, mask_size);
	assert ( (2*mask_size+1<=(signed) image.width) && (2*mask_size+1< (signed)image.height) );

	base::size buf_size = image.width*image.height;
	base::octet * buffer = new base::octet[buf_size];

	RGB2YCbCr(const_cast<common::Image &>(image)); // TODO: const_cast, berk
	base::octet * tmp_image = new base::octet [(image.width+mask_size*2) * (image.height+mask_size*2)];

	for (int i=0; i<3; ++i)
	{

		unsigned char * bp = setup_brightness_lut((i ? bt2:bt1), 4);

	    base::octet *p = image.buffer + i;

		for (base::offset o=0; o<buf_size; ++o, p+=3)
			buffer[o]=*p;
		int x_size = image.width;
		int y_size = image.height;
		enlarge(buffer,tmp_image,x_size,y_size,mask_size);

		susan_smoothing(pGaugeControler, i, image2.buffer+i, tmp_image, x_size, y_size, mask_size, bp, dp);

	}
	delete [] dp;
	delete [] tmp_image;
	delete [] buffer;
	YCbCr2RGB(image2);
}


