#include "stdafx.h"
#include "common.h"
#include "transform.h"
#include <cmath>
#include <iostream>


using namespace transform;



void getBounds(int count_needed, int * hist, base::octet & min, base::octet & max)
{
	int count=0;
	base::octet i=0;
	for (i=0; i<255; ++i)
	{
		if (count<count_needed)
		{
			count+=hist[i];
			if (count>=count_needed)
				break;
		}
	}
	min=i;


	count=0;
	for (i=255; i>0; --i)
	{
		if (count<count_needed)
		{
			count+=hist[i];
			if (count>=count_needed)
				break;

		}
	}
	max=i;

}

void AutoCorrect::Correct (const common::Image & in, common::Image & out, base::param br, base::param c) const
{
	base::octet * p1 = in.buffer;
	base::octet * p2 = out.buffer;

	int hist_r[256];
	int hist_g[256];
	int hist_b[256];

	memset(hist_r, 0, sizeof hist_r);
	memset(hist_g, 0, sizeof hist_g);
	memset(hist_b, 0, sizeof hist_b);

	// the following are used to record the point with the highest and lowest luminance.
	// this way, we can scale the luminance all by itself
	base::octet min_r=255,min_g=255,min_b=255,
				max_r=0,max_g=0,max_b=0;
	base::param min_luma = 1.0f, max_luma=0.0f;

	for (base::offset i=0; i<in.width*in.height; ++i)
	{
		base::octet r = *(p1++);
		base::octet g = *(p1++);
		base::octet b = *(p1++);
		hist_r[r]++;
		hist_g[g]++;
		hist_b[b]++;

		base::param luma = (0.299f*r + 0.587f*g + 0.114f*b)/255.0f;
		if (luma>max_luma)
		{
			max_luma=luma;
			max_r=r; 
			max_g=g; 
			max_b=b; 
		}
		if (luma<min_luma)
		{
			min_luma=luma;
			min_r=r;
			min_g=g;
			min_b=b;

		}
	}
	base::octet min_r2=255,min_g2=255,min_b2=255,
		max_r2=0,max_g2=0,max_b2=0;

	getBounds(in.width*in.height/500, hist_r, min_r2, max_r2);
	getBounds(in.width*in.height/500, hist_g, min_g2, max_g2);
	getBounds(in.width*in.height/500, hist_b, min_b2, max_b2);

	min_r=(base::octet)((min_r2*c+min_r*(1-c))*br);
	min_g=(base::octet)((min_g2*c+min_g*(1-c))*br);
	min_b=(base::octet)((min_b2*c+min_b*(1-c))*br);

	max_r=(base::octet)((max_r2*c+max_r*(1-c))*br+255*(1-br));
	max_g=(base::octet)((max_g2*c+max_g*(1-c))*br+255*(1-br));
	max_b=(base::octet)((max_b2*c+max_b*(1-c))*br+255*(1-br));

	p1 = in.buffer;

	for (base::offset i=0; i<in.width*in.height; ++i)
	{
		base::octet r = *(p1++);
		base::octet g = *(p1++);
      		base::octet b = *(p1++);
		  // base::param luma = (0.299f*r + 0.587f*g + 0.114f*b)/255.0f;

		*(p2++) = clamp((r-min_r)*255/(max_r-min_r));
		*(p2++) = clamp((g-min_g)*255/(max_g-min_g));
		*(p2++) = clamp((b-min_b)*255/(max_b-min_b));
	}
}





inline base::param getBCValue(base::param value, base::param brightness, base::param contrast)
{
float dc=value;


if (contrast<0.5)
  dc=(dc-0.5)*2*contrast + brightness;
else
  dc=(dc-0.5)/(2.01-contrast*2) + brightness;

return dc;
}


void BrightnessContrast::DoBC(const common::Image & in, common::Image & out, 
							  base::param brightness, base::param contrast, base::param saturation) const
{
	base::octet * p1 = in.buffer;
	base::octet * p2 = out.buffer;

	for (base::offset i=0; i<in.width*in.height; ++i, p1+=3, p2+=3)
	{
		base::octet r = p1[base::cRed], g=p1[base::cGreen], b=p1[base::cBlue]; //TODO: use base components
		base::param y = GetY(r,g,b);
		base::param newy = clamp(255.0*getBCValue(y/255.0, brightness, contrast));
		Recolor2(y, newy, saturation, r,g,b,true);

		p2[base::cRed] = r;  p2[base::cGreen] = g;  p2[base::cBlue] = b;
	}
}



base::octet BilinearResize::Bilinear(const base::octet * p, base::size width, 
									 base::octet orig_offset_x, base::octet orig_offset_y )
{
    base::octet r1 = p[0];
    base::octet r2 = p[3];
    base::octet rh = (r2*(orig_offset_x) + r1*(255-(orig_offset_x)))/256;

    r1= p[width];
    r2= p[width+3];
    base::octet rl = (r2*(orig_offset_x) + r1*(255-(orig_offset_x)))/256;
    base::octet r  = (rl*(orig_offset_y) + rh*(255-(orig_offset_y)))/256;        
    return r;    
}    


void BilinearResize::Resize(const common::Image & in, common::Image & out )
{   
     base::octet * pTarget = out.buffer;
     base::octet * p = in.buffer;
     
    base::size h = in.height;
    base::size w = in.width;

    base::size th = out.height;
    base::size tw = out.width;

     memset (pTarget, 240, th*tw*3);
 
 
    base::size orig_offset_y = 0;
    base::size delta_x = 256*(w-1)/tw;
    base::size delta_y = 256*(h-1)/th;
    
    size_t offset=0;
    for (base::offset y=0; y<th; ++y)
    {
        unsigned orig_offset_x=0;
        for (base::offset x=0; x<tw; ++x)
        {            
	  unsigned int orig_offset = ((orig_offset_y>>8)*w+(orig_offset_x>>8) )*3;

	  *pTarget++= Bilinear(p+orig_offset,   w*3, orig_offset_x&0xff, orig_offset_y&0xff);
	  *pTarget++= Bilinear(p+orig_offset+1, w*3, orig_offset_x&0xff, orig_offset_y&0xff);
	  *pTarget++= Bilinear(p+orig_offset+2, w*3, orig_offset_x&0xff, orig_offset_y&0xff);
	  offset+=3;

            orig_offset_x+=delta_x;
        }    
        orig_offset_y+=delta_y;
    }   
     
}

